@@ -4,8 +4,8 @@
  *       Sven Herzberg        <herzi@gnome-de.org>
  *       Adrien Beaucreux     <informancer@afturgurluk.org>
  *
- * Copyright (C) 2004 Sven Herzberg
- * Copyright (C) 2004 Adrien Beuacreux
+ * Copyright (C) 2004,2005 Sven Herzberg
+ * Copyright (C) 2004,2005 Adrien Beaucreux
  * Copyright (C) 2004 Keith Sharp
  *
  * This program is free software; you can redistribute it and/or
@@ -37,8 +37,12 @@
 #include <gnome.h>
 #include <glade/glade.h>
 
+#include <cdebug.h>
+#include <helpers/gtk-helpers.h>
+
+#include <utils/cria-lepton.h>
+
 #include "application.h"
-#include "debug.h"
 #include "libglade-support.h"
 #include "main-window.h"
 #include "slide-show.h"
@@ -46,7 +50,12 @@
 enum {
 	PROP_0,
 	PROP_DEFAULT_FONT,
-	PROP_GCONF_CLIENT
+	PROP_DETACH_MENUBARS,
+	PROP_DETACH_TOOLBARS,
+	PROP_DISABLE_SCREENSAVER,
+	PROP_GCONF_CLIENT,
+	PROP_PGDOWN_NEW_SLIDE,
+	PROP_SHOW_TEAROFF,
 };
 
 enum {
@@ -59,6 +68,8 @@ struct _CriaApplicationPriv {
 	EggRecentModel	     * recent_model;
 	PangoFontDescription * default_font;
 	GConfClient          * gconf_client;
+	gboolean	       disable_screensaver;
+	gboolean	       page_down_creates_slide;
 };
 
 /* method prototypes */
@@ -104,6 +115,11 @@ static void notify_default_font_changed        (GConfClient *client,
 
 static guint		  cria_application_signals[N_SIGNALS] = { 0 };
 
+static void
+cmw_about_uri_cb(GtkAboutDialog* d, const gchar* link, gpointer data) {
+	gnome_url_show(link, NULL);
+}
+
 /**
  * criawips_init:
  * @argc: pointer to argc as delivered to the main() method
@@ -114,15 +130,14 @@ static guint		  cria_application_signals[N_SIGNALS] = { 0 };
  */
 void
 criawips_init(int *argc, char ***argv) {
-#warning "criawips_init(): FIXME: check for a runnning instance, delegate command line tasks to it"
+#warning "init(): FIXME: check for a runnning instance, delegate command line tasks to it"
 	const gchar	**filenames;
 	GnomeProgram	* gnome_program;
 	GValue		* value;
 	poptContext	  popt_context;
-	GError		* error = NULL;
 
 	g_set_prgname (PACKAGE);
-	g_debug("criawips_init(): start");
+	cdebug("criawips_init()", "start");
 
 	/* initialize i18n */
 #ifdef ENABLE_NLS
@@ -135,30 +150,24 @@ criawips_init(int *argc, char ***argv) {
 	  /* GTK+ */
 	gtk_init (argc, argv);
 
-	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/pixmaps/criawips.png", &error);
-	if(error) {
-		gchar	* message = g_strdup_printf(_("Couldn't set window icon, you have to live without; reason: %s"), error->message);
-		cria_application_show_error_dialog(NULL, _("Error"), message);
-		g_error_free(error);
-		g_free(message);
-		error = NULL;
-	}
+	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/pixmaps/criawips.png", NULL);
+	gtk_about_dialog_set_url_hook(cmw_about_uri_cb, NULL, NULL);
 
 	  /* libgnome(ui) */
-	gnome_program = gnome_program_init (
-			PACKAGE, PACKAGE_VERSION,
-			LIBGNOMEUI_MODULE,
-			*argc, *argv,
-			GNOME_PARAM_POPT_TABLE, options,
-			GNOME_PARAM_POPT_FLAGS, 0,
-			NULL);
+	gnome_program = gnome_program_init(PACKAGE, PACKAGE_VERSION,
+					   LIBGNOMEUI_MODULE,
+					   *argc, *argv,
+					   GNOME_PARAM_POPT_TABLE, options,
+					   GNOME_PARAM_POPT_FLAGS, 0,
+					   GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
+					   GNOME_PARAM_HUMAN_READABLE_NAME, _("Criawips"),
+					   NULL);
 
 	  /* libglade */
 	glade_init();
 	glade_set_custom_handler(criawips_libglade_custom_handler, NULL);
 
-	/* libgoffice */
-	go_font_init();
+	cria_lepton_init();
 
 	value = g_new0(GValue, 1);
 	g_value_init(value,
@@ -169,11 +178,12 @@ criawips_init(int *argc, char ***argv) {
 
 	popt_context = (poptContext) g_value_get_pointer(value);
 	filenames = poptGetArgs(popt_context);
+	g_free(value);
 
 	cria_application_get_instance();
 
 	if(start_off) {
-		g_debug("criawips_init(): opening presentation mode");
+		cdebug("criawips_init()", "opening presentation mode");
 		
 		if(filenames && filenames[0]) {
 			if(filenames[1]) {
@@ -181,16 +191,12 @@ criawips_init(int *argc, char ***argv) {
 									  	      0,
 									  	      GTK_MESSAGE_WARNING,
 									  	      GTK_BUTTONS_CLOSE,
-									   	      "<span weight=\"bold\">%s</span>\n\n%s",
-									   	      _("Criawips can only show one presentation"),
-										      _("You tried to show more than one presentation. Criawips cannot handle this yet."));
+									   	      _("Criawips can only show one presentation"));
+				gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+									 _("You tried to show more than one presentation. Criawips cannot handle this yet."));
 				gtk_widget_show(dialog);
 				gtk_dialog_run(GTK_DIALOG(dialog));
-				gtk_widget_hide(dialog);
-
-				while(gtk_events_pending()) {
-					gtk_main_iteration();
-				}
+				gtk_widget_hide_now(dialog);
 				gtk_object_destroy(GTK_OBJECT(dialog));
 			}
 			
@@ -200,6 +206,7 @@ criawips_init(int *argc, char ***argv) {
 			GtkFileFilter	* filter;
 			gchar		* filename;
 
+#warning "init(): FIXME: extract the filechooser code into one code fragment"
 			dialog = gtk_file_chooser_dialog_new(_("Select a Presentation"),
 							     NULL,
 							     GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -213,10 +220,7 @@ criawips_init(int *argc, char ***argv) {
 						    filter);
 			switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
 			case GTK_RESPONSE_ACCEPT:
-				gtk_widget_hide(dialog);
-				while (gtk_events_pending()) {
-					gtk_main_iteration();
-				}
+				gtk_widget_hide_now(dialog);
 				filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 				cria_open_presentation_from_filename(filename);
 				g_free(filename);
@@ -231,23 +235,35 @@ criawips_init(int *argc, char ***argv) {
 			};
 		}
 	} else {
-		g_debug("criawips_init(): opening main window(s)");
+		cdebug("criawips_init()", "opening main window(s)");
 		
 		if(!filenames) {
 			/* Open the criawips main window here */
-			gtk_widget_show(GTK_WIDGET(cria_main_window_new(NULL)));
+			GtkWidget* mainwin = cria_main_window_new_default();
+			cria_main_window_set_geometry(CRIA_MAIN_WINDOW(mainwin));
 		} else {
 			/* open each file in a new window */
 			const gchar	**iterator;
-			g_debug("criawips_init(): filenames is a char** at 0x%x", (uintptr_t)filenames);
+			cdebug("criawips_init()", "filenames is a char** at 0x%x", (uintptr_t)filenames);
 			for(iterator = filenames; iterator && *iterator; iterator++) {
-				g_debug("criawips_init(): iterator is now 0x%x >> 0x%x (%s)", (uintptr_t)iterator, (uintptr_t)(*iterator), *iterator);
-				gtk_widget_show(GTK_WIDGET(cria_main_window_new(*iterator)));
+				cdebug("criawips_init()", "iterator is now 0x%x >> 0x%x (%s)", (uintptr_t)iterator, (uintptr_t)(*iterator), *iterator);
+				GtkWidget* mainwin = cria_main_window_new(*iterator);
+				cria_main_window_set_geometry(CRIA_MAIN_WINDOW(mainwin));
 			}
 		}
 	}
 
-	g_debug("criawips_init(): end");
+	cdebug("criawips_init()", "end");
+}
+
+/**
+ * criawips_shutdown:
+ *
+ * Shutdown all initialized systems.
+ */
+void
+criawips_shutdown(void) {
+	cria_lepton_shutdown_debug();
 }
 
 /**
@@ -256,14 +272,15 @@ criawips_init(int *argc, char ***argv) {
  * Quit Criawips, close all application windows and then exit the application.
  */
 void
-criawips_quit (void) {
+criawips_quit(void) {
+#warning "quit(): go through the windows an destroy one after the other"
 #warning "criawips_quit(): FIXME: Ask open windows to save their content"
 
-	/* FIXME: should be freed in the application destructor */
-        g_object_unref (G_OBJECT (self->priv->gconf_client));
+#warning "criawips_quit(): FIXME: should be freed in the application destructor"
+        g_object_unref(G_OBJECT(self->priv->gconf_client));
 
 	/* get the application object and quit all window instances */
-	if (gtk_main_level() > 0) {
+	if(gtk_main_level() > 0) {
 		gtk_main_quit ();
 	} else {
 		exit(0);
@@ -281,18 +298,17 @@ cria_open_presentation_from_filename(const gchar*filename) {
 	GError			* error = NULL;
 
 	g_assert(filename != NULL);
-	g_debug("Opening file '%s'", filename);
+	cdebug("cria_open_presentation_from_filename()", "Opening file '%s'", filename);
 
 	pres = cria_presentation_new_from_file(filename, &error);
 
 	if(error) {
-		char		* error_message = g_strdup_printf("<span weight=\"bold\">%s</span>\n\n%s %s:\n<span style=\"italic\">%s</span>.",
-								  _("The Presentation could not be opened"),
+		char		* error_message = g_strdup_printf("%s %s:\n<span style=\"italic\">%s</span>.",
 								  _("For some reason the presentation you wanted to open could not be opened."),
 								  _("The reason was:"),
 								  error->message);
 		cria_application_show_error_dialog(NULL,
-						   _("Error opening file"),
+						   _("The Presentation could not be opened"),
 						   error_message);
 		g_free(error_message);
 		g_error_free(error);
@@ -331,7 +347,6 @@ cria_application_class_init (CriaApplicationClass* cria_application_class) {
 	g_object_class->set_property = cria_application_set_property;
 	g_object_class->get_property = cria_application_get_property;
 
-	/* FIXME: We have runtime errors in the following function */
 	g_object_class_install_property(g_object_class,
 					PROP_DEFAULT_FONT,
 					g_param_spec_boxed("default_font",
@@ -339,41 +354,103 @@ cria_application_class_init (CriaApplicationClass* cria_application_class) {
 							   "The default font for the slides",
 							   PANGO_TYPE_FONT_DESCRIPTION, 
 							   G_PARAM_READWRITE));
-
+	g_object_class_install_property(g_object_class,
+					PROP_DETACH_MENUBARS,
+					g_param_spec_boolean("detach-menubars",
+							     "Detach Menubars",
+							     "Specify whether the menubars are detachable",
+							     FALSE,
+							     G_PARAM_READABLE));
+	g_object_class_install_property(g_object_class,
+					PROP_DETACH_TOOLBARS,
+					g_param_spec_boolean("detach-toolbars",
+							     "Detach Toolbars",
+							     "Specify whether the toolbars are detachable",
+							     FALSE,
+							     G_PARAM_READABLE));
+	g_object_class_install_property(g_object_class,
+					PROP_DISABLE_SCREENSAVER,
+					g_param_spec_boolean("disable-screensaver",
+							     "Disable Screensaver",
+							     "Specify whether the screensaver should be turned off for presentations",
+							     FALSE,
+							     G_PARAM_READWRITE));
+	g_object_class_install_property(g_object_class,
+					PROP_PGDOWN_NEW_SLIDE,
+					g_param_spec_boolean("pagedown-new-slides",
+							     "Page Down can create new slides",
+							     "Specify whether pressing page down on the last slide creates a new one",
+							     TRUE,
+							     G_PARAM_READWRITE));
 	g_object_class_install_property(g_object_class,
 					PROP_GCONF_CLIENT,
-					g_param_spec_object("gconf_client",
-							    "GConf_client",
+					g_param_spec_object("gconf-client",
+							    "GConf client",
 							    "The Gconfclient for the application",
 							    GCONF_TYPE_CLIENT,
 							    G_PARAM_READABLE));
+	g_object_class_install_property(g_object_class,
+					PROP_SHOW_TEAROFF,
+					g_param_spec_boolean("show-tearoff",
+							     "Show Tearoff",
+							     "Show Tearoff Menu Items",
+							     FALSE,
+							     G_PARAM_READABLE));
 }
 
 /**
  * cria_application_register_window:
  * @window: a main window to be registeres by the application
  *
- * Registers a #CriaMainWindow to the application. Thus it's being propted to
+ * Registers a #CriaMainWindow to the application. So it's being propted to
  * close the file before quitting.
  */
 void
 cria_application_register_window(CriaMainWindow	* window) {
 	CriaApplication	* self;
 
-	g_debug("cria_application_register_window(): start");
+	cdebugo(cria_application_get_instance(), "registerWindow()", "start");
 
-	g_return_if_fail(window != NULL);
-	g_assert(CRIA_IS_MAIN_WINDOW(window));
+	g_return_if_fail(CRIA_IS_MAIN_WINDOW(window));
 
 	self = cria_application_get_instance();
 
-	self->priv->windows = g_list_prepend(
-			self->priv->windows,
-			g_object_ref(window));
+	self->priv->windows = g_list_prepend(self->priv->windows, g_object_ref(window));
 
 	/*g_object_notify(self, "windows");*/
 
-	g_debug("cria_application_register_window(): end");
+	cdebugo(cria_application_get_instance(), "registerWindow()", "end");
+}
+
+/**
+ * cria_application_unregister_window:
+ * @window: a main window to be unregistered by the application
+ * 
+ * Unregisters a #CriaMainWindow from the application.
+ */
+void
+cria_application_unregister_window(CriaMainWindow *window) {
+	CriaApplication	* self;
+
+	cdebugo(cria_application_get_instance(), "cria_application_unregister_window()", "start" );
+
+	g_return_if_fail(CRIA_IS_MAIN_WINDOW(window));
+
+	self = cria_application_get_instance();
+	/* Remove the window from the windows list */
+	if(g_list_find(self->priv->windows, window)) {
+		self->priv->windows = g_list_remove(self->priv->windows, window);
+		g_object_unref(window);
+	} else {
+		cdebugo(cria_application_get_instance(), "unregisterWindow()", "Trying to unregister an unregistered window.");
+	}
+		
+	/* If the list is empty, quit the application */
+	if(self->priv->windows == NULL) {
+		criawips_quit();
+	}
+
+	cdebugo(cria_application_get_instance(), "cria_application_unregister_window()", "end" );
 }
 
 /**
@@ -393,30 +470,160 @@ cria_application_get_instance (void) {
 	return self;
 }
 
+gchar*
+cria_application_get_default_geometry(void) {
+	return gconf_client_get_string(self->priv->gconf_client, "/apps/criawips/ui/default_geometry", NULL);
+}
+
+gboolean
+cria_application_get_detach_menubars(void) {
+	return gconf_client_get_bool(self->priv->gconf_client, "/desktop/gnome/interface/menubar_detachable", NULL);
+}
+
+gboolean
+cria_application_get_detach_toolbars(void) {
+	return gconf_client_get_bool(self->priv->gconf_client, "/desktop/gnome/interface/toolbar_detachable", NULL);
+}
+
+gboolean
+cria_application_get_disable_screensaver(void) {
+	g_return_val_if_fail(CRIA_IS_APPLICATION(self), FALSE);
+	g_return_val_if_fail(self->priv, FALSE);
+
+	return self->priv->disable_screensaver;
+}
+
+gboolean
+cria_application_get_page_down_creates_slide(void) {
+	g_return_val_if_fail(CRIA_IS_APPLICATION(self), FALSE);
+	g_return_val_if_fail(self->priv, FALSE);
+
+	return self->priv->page_down_creates_slide;
+}
+
+gboolean
+cria_application_get_show_sidebar(void) {
+	return gconf_client_get_bool(self->priv->gconf_client, "/apps/criawips/ui/sidebar_visible", NULL);
+}
+
+gboolean
+cria_application_get_show_tearoff(void) {
+	return gconf_client_get_bool(self->priv->gconf_client, "/desktop/gnome/interface/menus_have_tearoff", NULL);
+}
+
+gint
+cria_application_get_sidebar_width(void) {
+	return gconf_client_get_int(self->priv->gconf_client, "/apps/criawips/ui/sidebar_width", NULL);
+}
+
+const gchar*
+cria_application_get_zoom(void) {
+	return gconf_client_get_string(self->priv->gconf_client, "/apps/criawips/ui/default_zoom", NULL);
+}
+
 static void
-cria_application_get_property (
-		GObject		* object,
-		guint		  prop_id,
-		GValue		* value,
-		GParamSpec	* param_spec)
-{
+cria_application_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* param_spec) {
 	CriaApplication	* self;
 
 	self = CRIA_APPLICATION (object);
 
-	switch (prop_id) {
+	switch(prop_id) {
 	case PROP_DEFAULT_FONT:
 		g_value_set_object(value, self->priv->default_font);
 		break;
+	case PROP_DISABLE_SCREENSAVER:
+		g_value_set_boolean(value, cria_application_get_disable_screensaver());
+		break;
+	case PROP_PGDOWN_NEW_SLIDE:
+		g_value_set_boolean(value, cria_application_get_page_down_creates_slide());
+		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (
-				object,
-				prop_id,
-				param_spec);
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, param_spec);
 		break;
 	}
 }
 
+void
+cria_application_set_disable_screensaver(gboolean disable_screensaver) {
+	g_return_if_fail(CRIA_IS_APPLICATION(self));
+	g_return_if_fail(self->priv);
+
+	if(self->priv->disable_screensaver == disable_screensaver) {
+		return;
+	}
+
+	self->priv->disable_screensaver = disable_screensaver;
+	gconf_client_set_bool(self->priv->gconf_client,
+			      PACKAGE_GCONF_PATH "/page_down_creates_slides",
+			      disable_screensaver,
+			      NULL);
+
+	g_object_notify(G_OBJECT(self), "disable-screensaver");
+}
+
+static void
+notify_disable_screensaver_changed(GConfClient* client, guint conxn_id, GConfEntry* entry, gpointer user_data) {
+	GError  * error = NULL;
+	gboolean  ds = gconf_client_get_bool(self->priv->gconf_client,
+					     PACKAGE_GCONF_PATH "/disable_screensaver",
+					     &error);
+
+	if(error) {
+		g_critical("%s", error->message);
+		g_error_free(error);
+	}
+	
+	cria_application_set_disable_screensaver(ds);
+}
+
+void
+cria_application_set_page_down_creates_slide(gboolean pdcs) {
+	g_return_if_fail(CRIA_IS_APPLICATION(self));
+	g_return_if_fail(self->priv);
+
+	if(self->priv->page_down_creates_slide == pdcs) {
+		return;
+	}
+
+	self->priv->page_down_creates_slide = pdcs;
+	gconf_client_set_bool(self->priv->gconf_client,
+			      PACKAGE_GCONF_PATH "/page_down_creates_slides",
+			      pdcs,
+			      NULL);
+	
+	g_object_notify(G_OBJECT(self), "pagedown-new-slides");
+}
+
+static void
+notify_page_down_creates_slides(GConfClient* client, guint conxn_id, GConfEntry* entry, gpointer user_data) {
+	GError  * error = NULL;
+	gboolean  val = gconf_client_get_bool(self->priv->gconf_client,
+					      PACKAGE_GCONF_PATH "/page_down_creates_slides",
+					      &error);
+
+	if(error) {
+		g_critical("%s", error->message);
+		g_error_free(error);
+	}
+
+	cria_application_set_page_down_creates_slide(val);
+}
+
+static void
+notify_show_tearoff(GConfClient* client, guint conxn_id, GConfEntry* entry, gpointer data) {
+	g_object_notify(G_OBJECT(self), "show-tearoff");
+}
+
+static void
+notify_detach_menubars(GConfClient* client, guint conxn_id, GConfEntry* entry, gpointer data) {
+	g_object_notify(G_OBJECT(self), "detach-menubars");
+}
+
+static void
+notify_detach_toolbars(GConfClient* client, guint conxn_id, GConfEntry* entry, gpointer data) {
+	g_object_notify(G_OBJECT(self), "detach-toolbars");
+}
+
 static void
 cria_application_init(CriaApplication* app) {
 	GError * error = NULL;
@@ -447,17 +654,19 @@ cria_application_init(CriaApplication* app) {
 			     &error);
 	if(error) {
 		gchar	* message = g_strdup_printf(_("Couldn't open preferences directory: %s"), error->message);
-		cria_application_show_error_dialog(NULL, _("Error"), message);
-		g_error_free(error);
+		cria_application_show_error_dialog(NULL, _("The preferences system could not be initialized"), message);
 		g_free(message);
+		g_error_free(error);
 		error = NULL;
 	}
 
-	/* Set the default font */
-	notify_default_font_changed(NULL,
-				    0,
-				    NULL,
-				    NULL);
+	/* Get the configuration */
+	notify_default_font_changed(NULL, 0, NULL, NULL);
+	notify_disable_screensaver_changed(NULL, 0, NULL, NULL);
+	notify_page_down_creates_slides(NULL, 0, NULL, NULL);
+	notify_show_tearoff(NULL, 0, NULL, NULL);
+	notify_detach_menubars(NULL, 0, NULL, NULL);
+	notify_detach_toolbars(NULL, 0, NULL, NULL);
 
 	/* Set up notifications */
 	gconf_client_notify_add(self->priv->gconf_client,
@@ -467,32 +676,95 @@ cria_application_init(CriaApplication* app) {
 				NULL,
 				&error);
 	if(error) { 
-		g_debug("CriaApplication::init(): Couldn't add gconf notification: '%s'", error->message);
+		cdebugo(self, "init()", "Couldn't add gconf notification: '%s'", error->message);
 		g_error_free(error);
 		error = NULL;
 	}
+	
+	gconf_client_notify_add(self->priv->gconf_client,
+			        PACKAGE_GCONF_PATH "/disable_screensaver",
+				notify_disable_screensaver_changed,
+				NULL,
+				NULL,
+				&error);
+	if(error) { 
+		cdebugo(self, "init()", "Couldn't add gconf notification: '%s'", error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+	
+	gconf_client_notify_add(self->priv->gconf_client,
+				PACKAGE_GCONF_PATH "/page_down_creates_slides",
+				notify_page_down_creates_slides,
+				NULL,
+				NULL,
+				&error);
+	if(error) { 
+		cdebugo(self, "init()", "Couldn't add gconf notification: '%s'", error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	gconf_client_notify_add(self->priv->gconf_client,
+				"/desktop/gnome/interface/menus_have_tearoff",
+				notify_show_tearoff,
+				NULL,
+				NULL,
+				NULL);
+	gconf_client_notify_add(self->priv->gconf_client,
+				"/desktop/gnome/interface/menubar_detachable",
+				notify_detach_menubars,
+				NULL,
+				NULL,
+				NULL);
+	gconf_client_notify_add(self->priv->gconf_client,
+				"/desktop/gnome/interface/toolbar_detachable",
+				notify_detach_toolbars,
+				NULL,
+				NULL,
+				NULL);
+}
+
+void
+cria_application_set_default_geometry(const gchar* geometry) {
+	g_return_if_fail(geometry && strlen(geometry));
+
+	gconf_client_set_string(self->priv->gconf_client, "/apps/criawips/ui/default_geometry", geometry, NULL);
+}
+
+void
+cria_application_set_show_sidebar(gboolean ssb) {
+	gconf_client_set_bool(self->priv->gconf_client, "/apps/criawips/ui/sidebar_visible", ssb, NULL);
+}
+
+void
+cria_application_set_sidebar_width(gint sidebar_width) {
+	gconf_client_set_int(self->priv->gconf_client, "/apps/criawips/ui/sidebar_width", sidebar_width, NULL);
+}
+
+void
+cria_application_set_zoom(const gchar* zoom) {
+	gconf_client_set_string(self->priv->gconf_client, "/apps/criawips/ui/default_zoom", zoom, NULL);
 }
 
 static void
-cria_application_set_property (
-		GObject		* object,
-		guint		  prop_id,
-		const	GValue	* value,
-		GParamSpec	* param_spec)
-{
+cria_application_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* param_spec) {
 	CriaApplication	* self;
 	
-	self = CRIA_APPLICATION (object);
+	self = CRIA_APPLICATION(object);
 	
-	switch (prop_id) {
+	switch(prop_id) {
 	case PROP_DEFAULT_FONT:
 		cria_application_set_default_font_name(g_value_get_string(value));
 		break;
+	case PROP_DISABLE_SCREENSAVER:
+		cria_application_set_disable_screensaver(g_value_get_boolean(value));
+		break;
+	case PROP_PGDOWN_NEW_SLIDE:
+		cria_application_set_page_down_creates_slide(g_value_get_boolean(value));
+		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (
-				object,
-				prop_id,
-				param_spec);
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, param_spec);
 		break;
 	}
 }
@@ -500,34 +772,28 @@ cria_application_set_property (
 /**
  * cria_application_show_error_dialog:
  * @parent: a #GtkWindow or NULL
- * @title: the dialog title
- * @message: the dialog message
+ * @primary: the primary dialog message, the short description
+ * @secondary: the secondary dialog message, a detailed description; may be NULL
  *
  * Shows an error dialog. To be used when fatal things happen (e.g. loading the
  * ui definitions from the glade file failed). @parent is used to specify a
  * window that this dialog is transient to.
  */
 void
-cria_application_show_error_dialog (GtkWindow* parent, gchar const* title, gchar const* message) {
-	GtkWidget	* dialog,
-			* label;
+cria_application_show_error_dialog(GtkWindow* parent, gchar const* primary, gchar const* secondary) {
+	GtkWidget	* dialog;
 
-	dialog = gtk_dialog_new_with_buttons(title,
-                                             parent,
-					     GTK_DIALOG_NO_SEPARATOR,
-					     GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-					     NULL);
-	label = gtk_label_new("");
-	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_FILL);
-	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-	gtk_label_set_markup(GTK_LABEL(label), message);
-	gtk_widget_show(label);
-
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
-			  label);
-	gtk_widget_show_all(dialog);
+	dialog = gtk_message_dialog_new(parent,
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_CLOSE,
+					"%s", primary);
+	if(secondary) {
+		gtk_message_dialog_format_secondary_markup(GTK_MESSAGE_DIALOG(dialog), "%s", secondary);
+	}
 	
 	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_hide_now(dialog);
 	gtk_object_destroy(GTK_OBJECT(dialog));
 }
 
@@ -584,23 +850,13 @@ cria_application_get_default_font(void) {
 
 void
 cria_application_set_default_font_name(const gchar* font) {
-
-	GError* error = NULL;
-
-	g_assert(font);
+	g_return_if_fail(font && strlen(font));
 
 	gconf_client_set_string(self->priv->gconf_client,
 				PACKAGE_GCONF_PATH "/default_fallback_font",
 				font,
-				&error);
-
-	if(error) {
-		gchar	* message = g_strdup_printf(_("Couldn't set default font: %s"), error->message);
-		cria_application_show_error_dialog(NULL, _("Error"), message);
-		g_error_free(error);
-		g_free(message);
-		error = NULL;
-	}
+				NULL);
+	/* notify event will get triggered automatically */
 }
 
 static void
@@ -614,7 +870,7 @@ notify_default_font_changed(GConfClient *client,
 
 	PangoFontDescription *old_default = NULL;
 	
-	g_debug("CriaApplication::notify_default_font_changed(): start");
+	cdebugo(cria_application_get_instance(), "notifyDefaultFontChanged()", "start");
 
 	old_default = self->priv->default_font;
 
@@ -622,11 +878,13 @@ notify_default_font_changed(GConfClient *client,
 				       PACKAGE_GCONF_PATH "/default_fallback_font",
 				       &error);
 	if(error) {  
-		g_debug("CriaApplication::init(): Couldn't load default fallback font: '%s'", error->message);
+		cdebugo(cria_application_get_instance(), "notifyDefaultFontChanged()", "Couldn't load default fallback font: '%s'", error->message);
 		g_error_free(error);
 		error = NULL;
+		self->priv->default_font = pango_font_description_from_string("Sans 12");
 	} else if(!font) {
-		g_debug("CriaApplication::init(): Fallback font was null.");
+		cdebugo(cria_application_get_instance(), "notifyDefaultFontChanged()", "Fallback font was null.");
+		self->priv->default_font = pango_font_description_from_string("Sans 12");
 	} else {
 		self->priv->default_font = pango_font_description_from_string(font);
 		if(old_default) {
@@ -635,5 +893,6 @@ notify_default_font_changed(GConfClient *client,
 		}
 		g_signal_emit(self, cria_application_signals[CHANGED_DEFAULT_FONT_SIGNAL], 0, NULL);
 	}
-	g_debug("CriaApplication::notify_default_font_changed(): end");
+	cdebugo(cria_application_get_instance(), "notifyDefaultFontChanged()", "end");
 }
+
