@@ -3,7 +3,7 @@
  * AUTHORS
  *       Sven Herzberg        <herzi@gnome-de.org>
  *
- * Copyright (C) 2004 Sven Herzberg
+ * Copyright (C) 2004,2005 Sven Herzberg
  * Copyright (C) 2004 Keith Sharp
  * Copyright (C) 2004 Adrien Beaucreux
  *
@@ -26,7 +26,7 @@
 #include <glib-object.h>
 #include <gconf/gconf-client.h>
 #include "main-window.h"
-#include "egg-recent.h"
+#include <recent-files/egg-recent-model.h>
 
 #include <goffice/utils/go-font.h>
 
@@ -41,21 +41,39 @@ typedef struct _CriaApplicationPriv CriaApplicationPriv;
 
 void		  criawips_init			       (int		* argc,
 							char	       ***argv);
+void              criawips_shutdown                    (void);
 void		  criawips_quit			       (void);
+
+gchar*		  cria_application_get_default_geometry(void);
+gboolean	  cria_application_get_detach_menubars (void);
+gboolean	  cria_application_get_detach_toolbars (void);
+gboolean	  cria_application_get_disable_screensaver(void);
 CriaApplication	* cria_application_get_instance	       (void);
+gboolean	  cria_application_get_page_down_creates_slide(void);
+gboolean	  cria_application_get_show_sidebar    (void);
+gboolean	  cria_application_get_show_tearoff    (void);
+gint		  cria_application_get_sidebar_width   (void);
+const gchar	* cria_application_get_zoom	       (void);
 GType		  cria_application_get_type	       (void);
 void		  cria_application_register_window     (CriaMainWindow	* window);
+void		  cria_application_unregister_window   (CriaMainWindow	* window);
+void		  cria_application_set_default_geometry(const gchar	* geometry);
+void		  cria_application_set_disable_screensaver(gboolean disable_screensaver);
+void		  cria_application_set_page_down_creates_slide(gboolean pdcs);
+void		  cria_application_set_show_sidebar    (gboolean	  ssb);
+void		  cria_application_set_sidebar_width   (gint		  sidebar_width);
+void		  cria_application_set_zoom	       (const gchar	* zoom);
 void		  cria_application_show_error_dialog   (GtkWindow	* parent,
-							gchar const	* title,
-							gchar const	* message);
-EggRecentModel	* cria_application_get_recent_model	(void);
-void		cria_application_add_to_recent	(const	gchar	* text_uri);
+							gchar const	* primary,
+							gchar const	* secondary);
+EggRecentModel	* cria_application_get_recent_model    (void);
+void		  cria_application_add_to_recent       (const	gchar	* text_uri);
 PangoFontDescription * cria_application_get_default_font(void);
 void                   cria_application_set_default_font_name(const gchar* font);
 
 #define CRIA_TYPE_APPLICATION			(cria_application_get_type ())
 #define CRIA_APPLICATION(object)		(G_TYPE_CHECK_INSTANCE_CAST((object), CRIA_TYPE_APPLICATION, CriaApplication))
-#define CRIA_APPLICATION_CLASS(klass)		(G_TYPE_CHACK_CLASS_CAST((klass), CRIA_TYPE_APPLICATION, CriaApplicationClass))
+#define CRIA_APPLICATION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), CRIA_TYPE_APPLICATION, CriaApplicationClass))
 #define CRIA_IS_APPLICATION(object)		(G_TYPE_CHECK_INSTANCE_TYPE((object), CRIA_TYPE_APPLICATION))
 #define CRIA_IS_APPLICATION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), CRIA_TYPE_APPLICATION))
 #define CRIA_APPLICATION_GET_CLASS(object)	(G_TYPE_INSTANCE_GET_CLASS((object), CRIA_TYPE_APPLICATION, CriaApplicationClass))
