@@ -3,7 +3,7 @@
  * AUTHORS
  *       Sven Herzberg        <herzi@gnome-de.org>
  *
- * Copyright (C) 2004 Sven Herzberg
+ * Copyright (C) 2004,2005 Sven Herzberg
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 #include <config.h>
 #endif
 
-#include "block.h"
+#include "block-priv.h"
 
 #include <inttypes.h>
 #include <string.h>
@@ -33,41 +33,35 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
-
 #include <goffice/utils/go-color.h>
 
-#include "debug.h"
+#include <cdebug.h>
+#include <dom/cria-format.h>
+#include <utils/cria-units.h>
+
+#include "application.h"
 #include "enumerations.h"
 
 enum {
 	PROP_0,
+	PROP_ALIGNMENT_HORIZONTAL,
+	PROP_ALIGNMENT_VERTICAL,
+	PROP_FONT_FAMILY,
+	PROP_FONT_SIZE,
 	PROP_NAME,
 	PROP_POS_LEFT,
 	PROP_POS_RIGHT,
 	PROP_POS_TOP,
 	PROP_POS_BOTTOM,
-	PROP_ALIGNMENT_HORIZONTAL,
-	PROP_ALIGNMENT_VERTICAL,
+	PROP_TEXT,
 };
 
 enum {
-	SIGNAL,
+	FORMAT_CHANGED,
 	N_SIGNALS
 };
 
-struct _CriaBlockPrivate {
-	gchar			* name,
-	// FIXME: get rid of this one once we start to work out the span/block code
-				* text;
-	GoRect			* position;
-	/* FIXME: port the code to GOFont later */
-	//const GOFont		* font;
-	PangoFontDescription	* font;
-	CriaAlignment		  alignment;
-	CriaVAlignment		  valignment;
-	CriaSlide		* slide;
-	GOColor			* color;
-};
+static GObjectClass* parent_class = NULL;
 
 static	void	cria_block_get_property	       (GObject		* object,
 						guint		  prop_id,
@@ -78,44 +72,34 @@ static	void	cria_block_set_property        (GObject		* object,
 						guint		  prop_id,
 						const	GValue	* value,
 						GParamSpec	* param_spec);
-#if 0
-/* enable these to add support for signals */
+
 static	guint	cria_block_signals[N_SIGNALS] = { 0 };
 
-static	void	cria_block_signal	       (CriaBlock	* self,
-						const	gchar	* string);
-#endif
+static void
+cb_format_changed(CriaBlock* self, CriaFormatDomain format) {}
+
+static void
+cb_finalize(GObject* object) {
+	CriaBlock* self = CRIA_BLOCK(object);
+
+	cria_lepton_unref(self->format);
+	self->format = 0;
+
+	parent_class->finalize(object);
+}
 
 static void
 cria_block_class_init (CriaBlockClass	* cria_block_class) {
 	GObjectClass	* g_object_class;
 
-#warning "static Block::init(): FIXME: make the default font configurable"
-	cria_block_class->default_font = pango_font_description_new();
-	pango_font_description_set_family(cria_block_class->default_font, "Sans");
-	pango_font_description_set_size(cria_block_class->default_font, 12*8);
+	parent_class = g_type_class_peek_parent(cria_block_class);
 
 	g_object_class = G_OBJECT_CLASS(cria_block_class);
-#if 0
-	/* setting up signal system */
-	cria_block_class->signal = cria_block_signal;
 
-	cria_block_signals[SIGNAL] = g_signal_new (
-			"signal",
-			CRIA_TYPE_BLOCK,
-			G_SIGNAL_RUN_LAST,
-			G_STRUCT_OFFSET (
-				CriaBlockClass,
-				signal),
-			NULL,
-			NULL,
-			g_cclosure_marshal_VOID__STRING,
-			G_TYPE_NONE,
-			0);
-#endif
 	/* setting up property system */
 	g_object_class->set_property = cria_block_set_property;
 	g_object_class->get_property = cria_block_get_property;
+	g_object_class->finalize = cb_finalize;
 
 	g_object_class_install_property(g_object_class,
 					PROP_ALIGNMENT_HORIZONTAL,
@@ -159,6 +143,13 @@ cria_block_class_init (CriaBlockClass	* cria_block_class) {
 							    G_MAXUINT64,
 							    0,
 							    G_PARAM_READWRITE));
+	g_object_class_install_property(g_object_class,
+					PROP_TEXT,
+					g_param_spec_string("text",
+							    "Text",
+							    "The text contained in this block",
+							    "",
+							    G_PARAM_READWRITE));
 	g_object_class_install_property(g_object_class,
 					PROP_POS_TOP,
 					g_param_spec_uint64("top",
@@ -176,15 +167,28 @@ cria_block_class_init (CriaBlockClass	* cria_block_class) {
 							  CRIA_TYPE_VALIGNMENT,
 							  CRIA_VALIGNMENT_UNSET,
 							  G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	/* setting up the block class */
+	cria_block_class->format_changed = cb_format_changed;
+	
+	cria_block_signals[FORMAT_CHANGED] = g_signal_new("format-changed",
+					CRIA_TYPE_BLOCK,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(CriaBlockClass, format_changed),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__FLAGS,
+					G_TYPE_NONE,
+					1,
+					CRIA_TYPE_FORMAT_DOMAIN);
 }
 
-static CriaBlock*
-get_template(CriaBlock* self) {
+CriaBlock*
+cria_block_get_template(CriaBlock* self) {
 	CriaSlide* master;
 	CriaBlock* template;
 	
-	master   = cria_slide_get_master_slide(self->priv->slide);
-	template = (master == NULL)?NULL:cria_slide_get_block(master, self->priv->name);
+	master   = cria_slide_get_master_slide(self->slide);
+	template = (master == NULL) ? NULL : cria_slide_get_block(master, self->name);
 
 	return template;
 }
@@ -201,16 +205,15 @@ CriaAlignment
 cria_block_get_alignment(CriaBlock* self) {
 	CriaBlock* template;
 	
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), CRIA_ALIGNMENT_UNSET);
 
-	template = get_template(self);
+	template = cria_block_get_template(self);
 
-	if(self->priv->alignment == CRIA_ALIGNMENT_UNSET && CRIA_IS_BLOCK(template)) {
+	if(self->alignment == CRIA_ALIGNMENT_UNSET && CRIA_IS_BLOCK(template)) {
 		return cria_block_get_alignment(template);
 	}
 
-	return self->priv->alignment;
+	return self->alignment;
 }
 
 /**
@@ -226,57 +229,26 @@ GOColor
 cria_block_get_color(CriaBlock* self) {
 	CriaBlock	* template;
 	
-	g_assert(CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), RGBA_BLACK);
 
-	template = get_template(self);
+	template = cria_block_get_template(self);
 
-	if(self->priv->color != NULL) {
-		g_debug("Block::getColor(): returning our own");
-		return *(self->priv->color);
-	} else if (CRIA_IS_BLOCK(template)) {
-		g_debug("Block::getColor(): returning the one from the template");
+	if(self->color != NULL) {
+		cdebugo(self, "getColor()", "returning our own");
+		return *(self->color);
+	} else if(CRIA_IS_BLOCK(template)) {
+		cdebugo(self, "getColor()", "returning the one from the template");
 		return cria_block_get_color(template);
 	} else {
-		/* FIXME: do we want to handle the default here or in the renderer? */
-		g_debug("Block::getColor(): returning black");
+#warning "getColor(): FIXME: do we want to handle the default here or in the renderer?"
+		cdebugo(self, "getColor()", "returning black");
 		return RGBA_BLACK;
 	}
 }
 
-/**
- * cria_block_get_font:
- * @self: a #CriaBlock
- *
- * Get the font used to render a #CriaBlock.
- *
- * Returns the font used to render this block, NULL if none is set.
- */
-const PangoFontDescription*
-cria_block_get_font(CriaBlock* self) {
-	const PangoFontDescription* font;
-	CriaBlock	    * template;
-	
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
-	g_assert(self->priv->font != NULL || CRIA_BLOCK_GET_CLASS(self)->default_font != NULL);
-
-	template = get_template(self);
-
-	if(self->priv->font) {
-		g_debug("Block::getFont(): returning own font");
-		font = self->priv->font;
-	} else if(template) {
-		g_debug("Block::getFont(): returning from template");
-		font = cria_block_get_font(template);
-	} else {
-		g_debug("Block::getFont(): returing from class");
-		font = CRIA_BLOCK_GET_CLASS(self)->default_font;
-	}
-
-	g_debug("Block::getFont(): returning %s %i", pango_font_description_get_family(font), pango_font_description_get_size(font) / PANGO_SCALE);
-
-	return font;
+CriaLepton
+cria_block_get_format(CriaBlock* self) {
+	return self->format;
 }
 
 /**
@@ -290,11 +262,10 @@ cria_block_get_font(CriaBlock* self) {
 const gchar*
 cria_block_get_markup(CriaBlock* self) {
 #warning "Block::getMarkup(): get rid of me"
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
-	g_assert(self->priv->text != NULL);
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), "");
+	g_return_val_if_fail(self->text, "");
 	
-	return self->priv->text;
+	return self->text;
 }
 
 /**
@@ -307,10 +278,9 @@ cria_block_get_markup(CriaBlock* self) {
  */
 const gchar*
 cria_block_get_name(CriaBlock* self) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), "");
 	
-	return self->priv->name;
+	return self->name;
 }
 
 /**
@@ -327,25 +297,24 @@ cria_block_get_position(CriaBlock* self) {
 	CriaSlide	* slide;
 	CriaBlock	* template;
 	
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
-	g_assert(CRIA_IS_SLIDE(self->priv->slide));
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), NULL);
+	g_return_val_if_fail(CRIA_IS_SLIDE(self->slide), NULL);
 
-	slide    = self->priv->slide;
-	template = get_template(self);
+	slide    = self->slide;
+	template = cria_block_get_template(self);
 
-	if(self->priv->position != NULL) {
-		g_debug("Block::getPosition(): returning own position");
+	if(self->position != NULL) {
+		cdebugo(self, "getPosition()", "returning own position");
 		retval = g_new0(GoRect,1);
-		retval->left   = self->priv->position->left;
-		retval->right  = self->priv->position->right;
-		retval->top    = self->priv->position->top;
-		retval->bottom = self->priv->position->bottom;
+		retval->left   = self->position->left;
+		retval->right  = self->position->right;
+		retval->top    = self->position->top;
+		retval->bottom = self->position->bottom;
 	} else if(CRIA_IS_SLIDE(slide) && CRIA_IS_BLOCK(template)) {
-		g_debug("Block::getPosition(): returning position from master block");
-		retval = template->priv->position;
+		cdebugo(self, "getPosition()", "returning position from master block");
+		retval = template->position;
 	} else {
-		g_debug("Block::getPosition(): returning NULL");
+		cdebugo(self, "getPosition()", "returning NULL");
 	}
 
 	return retval;
@@ -353,11 +322,11 @@ cria_block_get_position(CriaBlock* self) {
 
 static void
 cria_block_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* param_spec) {
-	CriaBlock	* self;
+	CriaBlock* self;
 
 	self = CRIA_BLOCK(object);
 
-	switch (prop_id) {
+	switch(prop_id) {
 	case PROP_ALIGNMENT_HORIZONTAL:
 		g_value_set_enum(value, cria_block_get_alignment(self));
 		break;
@@ -368,21 +337,19 @@ cria_block_get_property(GObject* object, guint prop_id, GValue* value, GParamSpe
 		g_value_set_string(value, cria_block_get_name(self));
 		break;
 	case PROP_POS_LEFT:
-		g_value_set_uint64(value, self->priv->position->left);
+		g_value_set_uint64(value, self->position->left);
 		break;
 	case PROP_POS_RIGHT:
-		g_value_set_uint64(value, self->priv->position->right);
+		g_value_set_uint64(value, self->position->right);
 		break;
 	case PROP_POS_TOP:
-		g_value_set_uint64(value, self->priv->position->top);
+		g_value_set_uint64(value, self->position->top);
 		break;
 	case PROP_POS_BOTTOM:
-		g_value_set_uint64(value, self->priv->position->bottom);
+		g_value_set_uint64(value, self->position->bottom);
 		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
-						  prop_id,
-						  param_spec);
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, param_spec);
 		break;
 	}
 }
@@ -426,44 +393,44 @@ CriaVAlignment
 cria_block_get_valignment(CriaBlock* self) {
 	CriaBlock* template;
 	
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_val_if_fail(CRIA_IS_BLOCK(self), CRIA_VALIGNMENT_UNSET);
 
-	template = get_template(self);
+	template = cria_block_get_template(self);
 
-	if(self->priv->valignment == CRIA_VALIGNMENT_UNSET && template) {
+	if(self->valignment == CRIA_VALIGNMENT_UNSET && template) {
 		return cria_block_get_valignment(template);
 	}
 
-	return self->priv->valignment;
+	return self->valignment;
 }
 
 static void
 cria_block_init(CriaBlock* self) {
-	self->priv = g_new0(CriaBlockPrivate,1);
-#warning "CriaBlock::init(): FIXME: add configurable default font/size"
-	self->priv->font = NULL;
-	self->priv->text = g_strdup("");
+	self->text = g_strdup("");
 }
 
 CriaBlock*
 cria_block_new(CriaSlide* slide, const gchar* name) {
 	CriaBlock	* self;
 
-	g_assert(CRIA_IS_SLIDE(slide));
+	g_return_val_if_fail(CRIA_IS_SLIDE(slide), NULL);
 	
 	self = g_object_new(CRIA_TYPE_BLOCK, "name", name, NULL);
-	self->priv->slide = g_object_ref(slide);
+	self->slide = slide;
+	g_object_add_weak_pointer(G_OBJECT(slide), (gpointer*)&(self->slide));
 	
 	return self;
 }
 
 void
 cria_block_set_alignment(CriaBlock* self, CriaAlignment align) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 
-	self->priv->alignment = align;
+	if(align == self->alignment) {
+		return;
+	}
+
+	self->alignment = align;
 
 	g_object_notify(G_OBJECT(self), "alignment");
 }
@@ -477,75 +444,186 @@ cria_block_set_alignment(CriaBlock* self, CriaAlignment align) {
  */
 void
 cria_block_set_color(CriaBlock* self, GOColor color) {
-	g_assert(CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 
-	if(self->priv->color == NULL) {
-		self->priv->color = g_new0(GOColor, 1);
+	if(self->color == NULL) {
+		self->color = g_new0(GOColor, 1);
 	}
 
-	*(self->priv->color) = color;
+	*(self->color) = color;
 
-#warning "Block::setColor(): to this GObject-ish"
+#warning "Block::setColor(): do this GObject-ish"
 }
 
 void
 cria_block_set_font_family(CriaBlock* self, const gchar* font) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	gchar     * new_format;
+	CriaLepton  other;
+	
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 
-	if(!self->priv->font) {
-		self->priv->font = pango_font_description_copy(CRIA_BLOCK_GET_CLASS(self)->default_font);
+	new_format = cria_format_set(self->format, "font-family", font);
+	other = cria_lepton_ref(new_format);
+	g_free(new_format);
+	
+	if(other != self->format) {
+		CriaLepton three = self->format;
+		self->format = other;
+		other = three;
+
+		g_signal_emit(self, cria_block_signals[FORMAT_CHANGED], 0, (CriaFormatFont | CriaFormatSize));
+	}
+	
+	cria_lepton_unref(other);
+}
+
+/**
+ * cria_block_set_font_size:
+ * @self: a #CriaBlock
+ * @size: a string specifying a font size
+ *
+ * Sets the font size.
+ */
+void
+cria_block_set_font_size(CriaBlock* self, const gchar* size) {
+	gchar     * new_format;
+	CriaLepton  other;
+	
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+
+	new_format = cria_format_set(self->format, "font-size", size);
+	other = cria_lepton_ref(new_format);
+	g_free(new_format);
+
+	if(other != self->format) {
+		CriaLepton three = self->format;
+		self->format = other;
+		other = three;
+
+		g_signal_emit(self, cria_block_signals[FORMAT_CHANGED], 0, (CriaFormatFont | CriaFormatSize));
 	}
 
-	pango_font_description_set_family(self->priv->font, font);
-	g_debug("Block::setFontFamily(): set to \"%s\" %i", pango_font_description_get_family(self->priv->font), pango_font_description_get_size(self->priv->font));
+	cria_lepton_unref(other);
+}
+
+/**
+ * cria_block_set_font_size_int:
+ * @self: a #CriaBlock
+ * @size: the font size (in MCS)
+ *
+ * Sets the font size in master coordinates.
+ */
+void
+cria_block_set_font_size_int(CriaBlock* self, gint size) {
+	gchar* new_size;
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+
+	new_size = g_strdup_printf("%dpx", size / MASTER_COORDINATES_PER_POINT);
+	cria_block_set_font_size(self, new_size);
+	g_free(new_size);
 }
 
 void
-cria_block_set_font_size(CriaBlock* self, gint size) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv);
+cria_block_set_font_weight(CriaBlock* self, const gchar* weight) {
+	static const gchar* const valid_values[] = {
+		"100", "200", "300",
+		"400", "500", "600",
+		"700", "800", "900",
+		"bold", "bolder",
+		"inherit",
+		"lighter", "normal"
+	}; /* according to http://www.w3.org/TR/REC-CSS2/fonts.html#font-styling */
+	guint      i;
+	
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+	g_return_if_fail(weight);
 
-	if(!self->priv->font) {
-		self->priv->font = pango_font_description_copy(CRIA_BLOCK_GET_CLASS(self)->default_font);
+	for(i = 0; i < sizeof(valid_values) && strcmp(valid_values[i], weight) < 1; i++) {
+		if(!strcmp(weight, valid_values[i])) {
+			gchar* new_format = cria_format_set(self->format, "font-weight", valid_values[i]);
+			CriaLepton new = cria_lepton_ref(new_format);
+
+			if(new != self->format) {
+				CriaLepton three = self->format;
+				self->format = new;
+				new = three;
+
+				g_signal_emit(self, cria_block_signals[FORMAT_CHANGED], 0, (CriaFormatFont | CriaFormatSize));
+			}
+			
+			cria_lepton_unref(new);
+			g_free(new_format);
+
+			return;
+		}
 	}
 
-	pango_font_description_set_size(self->priv->font, size);
-	g_debug("Block::setFontSize(): set to \"%s\" %i", pango_font_description_get_family(self->priv->font), pango_font_description_get_size(self->priv->font));
+	g_critical("Unknown font-weight value: %s", weight);
+}
+
+void
+cria_block_set_font_style(CriaBlock* self, const gchar* style) {
+	static const gchar* const valid_values[] = {
+		"inherit", "italic",
+		"normal", "oblique"
+	}; /* according to http://www.w3.org/TR/REC-CSS2/fonts.html#font-styling */
+	guint i;
+
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+	g_return_if_fail(style);
+
+	for(i = 0; i < sizeof(valid_values) && strcmp(valid_values[i], style) < 1; i++) {
+		if(!strcmp(style, valid_values[i])) {
+			gchar* new_format = cria_format_set(self->format, "font-style", valid_values[i]);
+			CriaLepton new = cria_lepton_ref(new_format);
+
+			if(new != self->format) {
+				CriaLepton three = self->format;
+				self->format = new;
+				new = three;
+
+				g_signal_emit(self, cria_block_signals[FORMAT_CHANGED], 0, (CriaFormatFont | CriaFormatSize));
+			}
+
+			cria_lepton_unref(new);
+			g_free(new_format);
+			return;
+		}
+	}
+
+	g_critical("Unknown font-style value: %s", style);
 }
 
 void
 cria_block_set_name(CriaBlock* self, const gchar* name) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(name != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+	g_return_if_fail(name && *name);
 
-	if(self->priv->name && !strcmp(name, self->priv->name)) {
+	if(self->name && !strcmp(name, self->name)) {
 		return;
 	}
 
-	if (self->priv->name != NULL) {
-		g_free(self->priv->name);
+	if (self->name != NULL) {
+		g_free(self->name);
 	}
 
-	self->priv->name = g_strdup(name);
+	self->name = g_strdup(name);
 
 	g_object_notify(G_OBJECT(self), "name");
 }
 
 static void
 cria_block_set_pos_bottom(CriaBlock* self, go_unit_t bottom) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 
-	g_debug("Block::setPosBottom(): %lli", bottom);
+	cdebugo(self, "setPosBottom()", "%lli", bottom);
 	
-	if(self->priv->position == NULL) {
-		self->priv->position = g_new0(GoRect,1);
+	if(self->position == NULL) {
+		self->position = g_new0(GoRect,1);
 	}
 
-	if(bottom != self->priv->position->bottom) {
-		self->priv->position->bottom = bottom;
+	if(bottom != self->position->bottom) {
+		self->position->bottom = bottom;
 
 		g_object_notify(G_OBJECT(self), "bottom");
 	}
@@ -553,17 +631,16 @@ cria_block_set_pos_bottom(CriaBlock* self, go_unit_t bottom) {
 
 static void
 cria_block_set_pos_left(CriaBlock* self, go_unit_t left) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 	
-	g_debug("Block::setPosLeft(): %lli", left);
+	cdebugo(self, "setPosLeft()", "%lli", left);
 	
-	if(self->priv->position == NULL) {
-		self->priv->position = g_new0(GoRect,1);
+	if(self->position == NULL) {
+		self->position = g_new0(GoRect,1);
 	}
 
-	if(left != self->priv->position->left) {
-		self->priv->position->left = left;
+	if(left != self->position->left) {
+		self->position->left = left;
 
 		g_object_notify(G_OBJECT(self), "left");
 	}
@@ -571,17 +648,16 @@ cria_block_set_pos_left(CriaBlock* self, go_unit_t left) {
 
 static void
 cria_block_set_pos_right(CriaBlock* self, go_unit_t right) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 	
-	g_debug("Block::setPosRight(): %lli", right);
+	cdebugo(self, "setPosRight()", "%lli", right);
 	
-	if(self->priv->position == NULL) {
-		self->priv->position = g_new0(GoRect,1);
+	if(self->position == NULL) {
+		self->position = g_new0(GoRect,1);
 	}
 
-	if(right != self->priv->position->right) {
-		self->priv->position->right = right;
+	if(right != self->position->right) {
+		self->position->right = right;
 
 		g_object_notify(G_OBJECT(self), "right");
 	}
@@ -589,17 +665,16 @@ cria_block_set_pos_right(CriaBlock* self, go_unit_t right) {
 
 static void
 cria_block_set_pos_top(CriaBlock* self, go_unit_t top) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 	
-	g_debug("Block::setPosTop(): %lli", top);
+	cdebugo(self, "setPosTop()", "%lli", top);
 	
-	if(self->priv->position == NULL) {
-		self->priv->position = g_new0(GoRect,1);
+	if(self->position == NULL) {
+		self->position = g_new0(GoRect,1);
 	}
 
-	if(top != self->priv->position->top) {
-		self->priv->position->top = top;
+	if(top != self->position->top) {
+		self->position->top = top;
 
 		g_object_notify(G_OBJECT(self), "top");
 	}
@@ -607,11 +682,11 @@ cria_block_set_pos_top(CriaBlock* self, go_unit_t top) {
 
 static void
 cria_block_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* param_spec) {
-	CriaBlock	* self;
+	CriaBlock* self;
 	
-	self = CRIA_BLOCK (object);
+	self = CRIA_BLOCK(object);
 	
-	switch (prop_id) {
+	switch(prop_id) {
 	case PROP_ALIGNMENT_HORIZONTAL:
 		cria_block_set_alignment(self, g_value_get_enum(value));
 		break;
@@ -634,9 +709,7 @@ cria_block_set_property(GObject* object, guint prop_id, const GValue* value, GPa
 		cria_block_set_pos_bottom(self, g_value_get_uint64(value));
 		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
-						  prop_id,
-						  param_spec);
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, param_spec);
 		break;
 	}
 }
@@ -644,22 +717,27 @@ cria_block_set_property(GObject* object, guint prop_id, const GValue* value, GPa
 void
 cria_block_set_text(CriaBlock* self, const gchar* text) {
 #warning "Block::setText(): get rid of me"
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(text != NULL && strlen(text));
+	g_return_if_fail(CRIA_IS_BLOCK(self));
+	g_return_if_fail(text != NULL && *text);
 
-	if(self->priv->text) {
-		g_free(self->priv->text);
+	if(!strcmp(text, self->text)) {
+		return;
+	}
+
+	if(self->text) {
+		g_free(self->text);
 	}
 	
-	self->priv->text = g_strdup(text);
+	self->text = g_strdup(text);
+
+	g_object_notify(G_OBJECT(self), "text");
 }
 
 void
 cria_block_set_valignment(CriaBlock* self, CriaVAlignment valign) {
-	g_assert(self != NULL && CRIA_IS_BLOCK(self));
-	g_assert(self->priv != NULL);
+	g_return_if_fail(CRIA_IS_BLOCK(self));
 	
-	self->priv->valignment = valign;
+	self->valignment = valign;
 
 	g_object_notify(G_OBJECT(self), "valignment");
 }
