@@ -24,20 +24,15 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#include <cdebug.h>
+
 #include "application.h"
-#include "debug.h"
 
 gint
-main (
-		gint	  argc,
-		gchar	**argv)
-{
+main(gint argc, gchar** argv) {
 	criawips_init(&argc, &argv);
-
-	g_debug("main(): entering loop");
-	gtk_main ();
-
-	g_debug("main(): quitting");
-	go_font_shutdown();
+	gtk_main();
+	criawips_shutdown();
+	
 	return 0;
 }
