@@ -3,7 +3,7 @@
  * AUTHORS
  *       Sven Herzberg        <herzi@gnome-de.org>
  *
- * Copyright (C) 2004 Sven Herzberg
+ * Copyright (C) 2004,2005 Sven Herzberg
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,11 +36,12 @@
 #include <gsf/gsf-libxml.h>
 #include <goffice/utils/go-color.h>
 
-#include "debug.h"
-#include "helpers/gsf-pixbuf.h"
+#include <cdebug.h>
+#include <utils/cria-units.h>
 
 typedef struct {
 	GList			* tags;
+	GnomeVFSURI		* uri;
 	guint			  unknown;
 	GString			* cdata;
 	GError			* error;
@@ -65,43 +66,50 @@ static guint hex_to_int(gchar ch);
 
 /**
  * cria_presentation_populate_from_xml_input:
- * @self: the presentation that's to be filled from the data
- * @input: the GsfInput used to fill the presentation
- * @error: a location to return a GError
+ * @input: the #GsfInput used to fill the presentation
+ * @uri: the #GnomeVFSURI for this file
+ * @error: a location to return a #GError
  *
  * Takes an empty presentation and fills it with the data from the input
+ *
+ * Returns a new presentation from the #GsfInput
  */
-void
-cria_presentation_populate_from_xml_input(CriaPresentation* self, GsfInput *input, GError**error) {
+CriaPresentation*
+cria_presentation_populate_from_xml_input(GsfInput* input, GnomeVFSURI* uri, GError**error) {
 	xmlParserCtxt	* context;
 	Sax001Data	* data;
+	CriaPresentation* self;
 	
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
 #warning "Presentation::populateFromXML(): check for empty presentation"
-	g_assert(*error == NULL);
-	g_debug("Presentation::populateFromXML(): start");
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+	cdebugt(CRIA_TYPE_PRESENTATION, "populateFromXML()", "start");
 
 	context = gsf_xml_parser_context(input);
 	context->sax = &cria_0_0_1_handler;
 	data = g_new0(Sax001Data, 1);
-	data->tags = g_list_append(data->tags, self);
-	g_debug("Presentation::populateFromXML(): %d elements on the stack", g_list_length(data->tags));
+	data->uri = gnome_vfs_uri_ref(uri);
+	cdebugt(CRIA_TYPE_PRESENTATION, "populateFromXML()", "%d elements on the stack", g_list_length(data->tags));
 	context->userData = data;
 
 #warning "Presentation::populateFromXML(): FIXME: validate against DTD or Schema"
 	xmlParseDocument(context);
 
+	self = CRIA_PRESENTATION(data->tags->data);
+
 	if(data->error) {
 		*error = data->error;
 	} else if(context->wellFormed) {
-		g_debug("Presentation::populateFromXML(): really good file");
+		cdebugo(self, "populateFromXML()", "really good file");
 	} else {
-		g_debug("Presentation::populateFromXML(): really bad file, we haven't gotten an error");
+		cdebugo(self, "populateFromXML()", "really bad file, we haven't gotten an error");
 		*error = g_error_new(g_quark_from_static_string("criawips"), 1, "Unknown error: please file a bug report");
 	}
 
+	gnome_vfs_uri_unref(data->uri);
 	g_free(data);
-	g_debug("Presentation::populateFromXML(): end");
+	cdebugo(self, "populateFromXML()", "end");
+
+	return self;
 }
 
 static void
@@ -117,10 +125,10 @@ cria_sax_0_0_1_characters (
 		return;
 	}
 
-	str = g_strndup (string, length);
-	g_strstrip (str);
+	str = g_strndup(string, length);
+	g_strstrip(str);
 	
-	if (strlen (str)) {
+	if(strlen(str)) {
 		/* just do this if we really have some data to care about */
 		char	**strv,
 			**iterator;
@@ -134,26 +142,27 @@ cria_sax_0_0_1_characters (
 		 * 4. cut leading space if cdata was empty
 		 */
 		/* split the string into single lines to be processed */
-		strv = g_strsplit (
-				str,
-				"\n",
-				0);
+		strv = g_strsplit(str, "\n", 0);
 
-		if (!data->cdata) {
-			data->cdata = g_string_new ("");
+		if(!data->cdata) {
+			data->cdata = g_string_new("");
 		}
 
-		for (iterator = strv; *iterator; iterator++) {
-			g_strstrip (*iterator);
+		for(iterator = strv; *iterator; iterator++) {
+			g_strstrip(*iterator);
 
 			/* append one whitespace between lines */
 			/* but don't do if we already have '\n' there */
 /*			if (strlen (data->cdata->str) && (data->cdata->str[strlen (data->cdata->str) - 1] != '\n'))
 				g_string_append (data->cdata, " ");
 */
-			g_string_append (data->cdata, *iterator);
+			g_string_append(data->cdata, *iterator);
 		}
+
+		g_strfreev(strv);
 	}
+
+	g_free(str);
 }
 
 static void
@@ -183,51 +192,13 @@ cria_sax_0_0_1_element_background_end(Sax001Data* data) {
 	g_assert(CRIA_IS_IMAGE(g_list_last(data->tags)->data));
 	g_assert(CRIA_IS_SLIDE(g_list_nth(data->tags, g_list_length(data->tags) - 2)->data));
 	
-	cria_slide_set_background(g_list_nth(data->tags, g_list_length(data->tags) - 2)->data,
+	cria_background_set_image(cria_slide_get_background(g_list_nth(data->tags, g_list_length(data->tags) - 2)->data, TRUE),
 				  CRIA_IMAGE(g_list_last(data->tags)->data));
 
 	data->tags = g_list_remove(data->tags,
 				   g_list_last(data->tags)->data);
 }
 
-static go_unit_t
-get_master_coordinate(const gchar* string, go_unit_t presentation_size) {
-	gchar	 * endchar;
-	go_unit_t retval;
-	
-	g_debug("getMasterCoordinate(): parsing \"%s\"", string);
-
-	if(!string) {
-		return 0;
-	}
-	
-	retval = strtoull(string, &endchar, 10);
-	g_strstrip(endchar);
-
-	/* detect the unit */
-	if(endchar == NULL || strlen(endchar) == 0 || !strcmp(endchar, "px")) {
-		/* points */
-		retval = GO_PT_TO_IN(retval*576);
-	} else if(!strcmp(endchar, "cm") || !strcmp(endchar, "mm")) {
-		/* cm/mm */
-		if(*endchar == 'c') {
-			retval *= 10;
-		}
-		retval = GO_CM_TO_IN(retval*576);
-	} else if(!strcmp(endchar, "in")) {
-		/* inch */
-		retval = retval*576;
-	} else if(!strcmp(endchar, "%")) {
-		/* percent */
-		retval = retval*presentation_size/100;
-	} else {
-		/* default to points */
-		retval = GO_PT_TO_IN(retval*576);
-	}
-
-	return retval;
-}
-
 static guint
 hex_to_int(gchar ch) {
 	if('0' <= ch && ch <= '9') {
@@ -259,8 +230,8 @@ cria_sax_0_0_1_element_block_start(GHashTable* hash_table, Sax001Data* data) {
 	attr1 = g_hash_table_lookup(hash_table, "x");
 	attr2 = g_hash_table_lookup(hash_table, "width");
 	if((attr1 != NULL && strlen(attr1)) || (attr2 != NULL && strlen(attr2))) {
-		go_unit_t	x = get_master_coordinate(attr1, slide_size->x),
-				w = get_master_coordinate(attr2, slide_size->x);
+		go_unit_t	x = 0.0 + cria_unit_to_pixels(attr1, slide_size->x / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT,
+				w = 0.0 + cria_unit_to_pixels(attr2, slide_size->x / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT;
 		
 		g_object_set(G_OBJECT(block),
 			     "left", x,
@@ -271,8 +242,8 @@ cria_sax_0_0_1_element_block_start(GHashTable* hash_table, Sax001Data* data) {
 	attr1 = g_hash_table_lookup(hash_table, "y");
 	attr2 = g_hash_table_lookup(hash_table, "height");
 	if((attr1 != NULL && strlen(attr1)) || (attr2 != NULL && strlen(attr2))) {
-		go_unit_t	y = get_master_coordinate(attr1, slide_size->y),
-				h = get_master_coordinate(attr2, slide_size->y);
+		go_unit_t	y = 0.0 + cria_unit_to_pixels(attr1, slide_size->y / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT,
+				h = 0.0 + cria_unit_to_pixels(attr2, slide_size->y / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT;
 		
 		g_object_set(G_OBJECT(block),
 			     "top", y,
@@ -284,13 +255,16 @@ cria_sax_0_0_1_element_block_start(GHashTable* hash_table, Sax001Data* data) {
 	attr1 = g_hash_table_lookup(hash_table, "family");
 	attr2 = g_hash_table_lookup(hash_table, "size");
 	if(attr1 != NULL && strlen(attr1)) {
-		g_debug("PresentationParser::elementBlockStart(): setting font family \"%s\"", attr1);
+		cdebug("PresentationParser::elementBlockStart()", "setting font family \"%s\"", attr1);
 		cria_block_set_font_family(block, attr1);
 	}
 	
 	if(attr2 != NULL && strlen(attr2)) {
-		g_debug("PresentationParser::elementBlockStart(): setting font size %lli (units); %i (int)", get_master_coordinate(attr2, slide_size->y), (int)get_master_coordinate(attr2, slide_size->y));
-		cria_block_set_font_size(block, get_master_coordinate(attr2, slide_size->y));
+		cdebug("PresentationParser::elementBlockStart()",
+		       "setting font size %lli (units); %i (int)",
+		       cria_unit_to_pixels(attr2, slide_size->y / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT,
+		       (int)cria_unit_to_pixels(attr2, slide_size->y / MASTER_COORDINATES_PER_POINT));
+		cria_block_set_font_size_int(block, cria_unit_to_pixels(attr2, slide_size->y / MASTER_COORDINATES_PER_POINT) * MASTER_COORDINATES_PER_POINT);
 	}
 
 	/* colors */
@@ -299,14 +273,13 @@ cria_sax_0_0_1_element_block_start(GHashTable* hash_table, Sax001Data* data) {
 		guint r, g, b;
 		GOColor color;
 
-		// the first one is a '#'
-
+		/* the first one is a '#' */
 		r = hex_to_int(attr1[1]) * 16 + hex_to_int(attr1[2]);
 		g = hex_to_int(attr1[3]) * 16 + hex_to_int(attr1[4]);
 		b = hex_to_int(attr1[5]) * 16 + hex_to_int(attr1[6]);
 
 		color = RGBA_TO_UINT(r, g, b, 0xff);
-		g_debug("PresentationParser::elementBlockStart(): got \"%s\" as color, interpreted as 0x%x", attr1, color);
+		cdebug("PresentationParser::elementBlockStart()", "got \"%s\" as color, interpreted as 0x%x", attr1, color);
 
 		cria_block_set_color(block, color);
 	}
@@ -348,24 +321,24 @@ cria_sax_0_0_1_element_block_end(Sax001Data* data) {
 static void
 cria_sax_0_0_1_element_br_end (Sax001Data* data) {
 	if (data->cdata) {
-		//g_message("cria_sax_0_0_1_element_br_end(): data->cdata = 0x%x (%s)", (uintptr_t)data->cdata, (data->cdata==NULL)?NULL:data->cdata->str);
+		/*g_message("cria_sax_0_0_1_element_br_end(): data->cdata = 0x%x (%s)", (uintptr_t)data->cdata, (data->cdata==NULL)?NULL:data->cdata->str);*/
 		g_string_append(data->cdata, "\n");
 	}
 }
 
 static void
 cria_sax_0_0_1_element_description_start(GHashTable* hash_table, Sax001Data* data) {
-	g_debug("cria_sax_0_0_1_element_description_start(): start");
+	cdebug("cria_sax_0_0_1_element_description_start()", "start");
 	data->tags = g_list_append(data->tags,
 				   g_strdup("description"));
-	g_debug("cria_sax_0_0_1_element_description_start(): end");
+	cdebug("cria_sax_0_0_1_element_description_start()", "end");
 }
 
 static void
 cria_sax_0_0_1_element_description_end(Sax001Data* data) {
-	g_debug("cria_sax_0_0_1_element_description_end(): start");
+	cdebug("cria_sax_0_0_1_element_description_end()", "start");
 
-	/* FIXME are we leaking memory here? */
+	g_free(g_list_last(data->tags)->data);
 	data->tags = g_list_remove(data->tags,
 				   g_list_last(data->tags)->data);
 
@@ -374,21 +347,21 @@ cria_sax_0_0_1_element_description_end(Sax001Data* data) {
 
 	g_string_free(data->cdata, TRUE);
 	data->cdata = NULL;
-	g_debug("cria_sax_0_0_1_element_description_end(): end");
+	cdebug("cria_sax_0_0_1_element_description_end()", "end");
 }
 
 static void
 cria_sax_0_0_1_element_layout_start(GHashTable* hash_table, Sax001Data* data) {
 	CriaSlide * master_slide;
 	
-	g_debug ("cria_sax_0_0_1_element_layout_start(): begin");
+	cdebug("cria_sax_0_0_1_element_layout_start()", "start");
 
 	master_slide = cria_slide_new(NULL);
 	cria_slide_set_title(master_slide, g_hash_table_lookup(hash_table, "name"));
 	data->tags = g_list_append(data->tags,
 				   master_slide);
 	
-	g_debug ("cria_sax_0_0_1_element_layout_start(): end");
+	cdebug("cria_sax_0_0_1_element_layout_start()", "end");
 }
 
 static void
@@ -396,7 +369,7 @@ cria_sax_0_0_1_element_layout_end(Sax001Data* data) {
 	CriaTheme	* theme = g_list_nth(data->tags, g_list_length(data->tags) - 2)->data;
 	CriaSlide	* slide = g_list_last(data->tags)->data;
 	
-	g_debug ("cria_sax_0_0_1_element_layout_end(): begin");
+	cdebug("cria_sax_0_0_1_element_layout_end()", "begin");
 	
 	g_assert(theme != NULL && CRIA_IS_THEME(theme));
 	g_assert(slide != NULL && CRIA_IS_SLIDE(slide));
@@ -405,19 +378,52 @@ cria_sax_0_0_1_element_layout_end(Sax001Data* data) {
 
 	data->tags = g_list_remove(data->tags, slide);
 
-	g_debug ("cria_sax_0_0_1_element_layout_end(): end");
+	cdebug("cria_sax_0_0_1_element_layout_end()", "end");
+}
+
+static gboolean
+is_true(const gchar* string) {
+	g_return_val_if_fail(string, FALSE);
+	if (!strcmp(string, "true") || !strcmp(string, "yes")) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
 }
 
 static void
 cria_sax_0_0_1_element_presentation_start(GHashTable* hash_table, Sax001Data* data) {
-	g_debug ("cria_sax_element_presentation_start(): start");
-	g_assert(data != NULL);
-	g_assert(1 == g_list_length(data->tags));
-	g_assert(CRIA_IS_PRESENTATION(data->tags->data));
+	gchar* attribute;
 	
-	cria_presentation_set_title(CRIA_PRESENTATION(data->tags->data), g_hash_table_lookup(hash_table, "title"));
+	cdebug("cria_sax_element_presentation_start()", "start");
+	g_return_if_fail(data != NULL);
+	g_return_if_fail(0 == g_list_length(data->tags));
+
+	attribute = g_hash_table_lookup(hash_table, "title");
+	if(attribute) {
+		data->tags = g_list_append(data->tags, g_object_new(CRIA_TYPE_PRESENTATION, "title", attribute, NULL));
+	} else {
+		data->tags = g_list_append(data->tags, g_object_new(CRIA_TYPE_PRESENTATION, NULL));
+	}
+	cria_presentation_set_uri(CRIA_PRESENTATION(data->tags->data), data->uri);
+
+	attribute = g_hash_table_lookup(hash_table, "auto-switch");
+	if(attribute) {
+		cria_presentation_set_auto_switch(CRIA_PRESENTATION(data->tags->data), is_true(attribute));
+	}
+
+	attribute = g_hash_table_lookup(hash_table, "delay");
+	if(attribute) {
+		gdouble delay = g_ascii_strtod(attribute, NULL);
+		cria_presentation_set_slide_delay(CRIA_PRESENTATION(data->tags->data), delay);
+	}
+
+	attribute = g_hash_table_lookup(hash_table, "loop");
+	if(attribute) {
+		cria_presentation_set_loop(CRIA_PRESENTATION(data->tags->data), is_true(attribute));
+	}
 	
-	g_debug ("cria_sax_element_presentation_start(): end");
+	cdebug("cria_sax_element_presentation_start()", "end");
 }
 
 static void
@@ -434,11 +440,12 @@ cria_sax_0_0_1_element_slide_start(GHashTable* hash_table, Sax001Data* data) {
 	CriaSlide	* slide;
 	CriaTheme	* theme;
 	
-	g_assert(g_hash_table_lookup(hash_table, "title") != NULL);
-	g_assert(g_hash_table_lookup(hash_table, "theme") != NULL);
-	g_assert(g_hash_table_lookup(hash_table, "layout")!= NULL);
+#warning "elementSlideStart(): FIXME: add a more verbose error message"
+	g_return_if_fail(g_hash_table_lookup(hash_table, "title") != NULL);
+	g_return_if_fail(g_hash_table_lookup(hash_table, "theme") != NULL);
+	g_return_if_fail(g_hash_table_lookup(hash_table, "layout")!= NULL);
 
-	slide = cria_slide_new(CRIA_PRESENTATION(data->tags->data));
+	slide = cria_slide_new(CRIA_SLIDE_LIST(data->tags->data));
 	cria_slide_set_title(slide, g_hash_table_lookup(hash_table, "title"));
 	presentation = CRIA_PRESENTATION(data->tags->data);
 	theme = cria_presentation_get_theme(presentation, g_hash_table_lookup(hash_table, "theme"));
@@ -459,16 +466,15 @@ cria_sax_0_0_1_element_slide_end(Sax001Data* data) {
 
 static void
 cria_sax_0_0_1_element_theme_start(GHashTable* hash_table, Sax001Data* data) {
-	g_debug ("cria_sax_0_0_1_element_theme_start(): start");
+	cdebug("cria_sax_0_0_1_element_theme_start()", "start");
 	data->tags = g_list_append(data->tags,
 				   cria_theme_new(g_hash_table_lookup(hash_table, "name")));
-	g_debug ("cria_sax_0_0_1_element_theme_start(): end");
+	cdebug("cria_sax_0_0_1_element_theme_start()", "end");
 }
 
 static void
-cria_sax_0_0_1_element_theme_end (Sax001Data* data)
-{
-	g_debug ("cria_sax_0_0_1_element_theme_end(): start");
+cria_sax_0_0_1_element_theme_end (Sax001Data* data) {
+	cdebug("cria_sax_0_0_1_element_theme_end()", "start");
 	g_return_if_fail (CRIA_IS_PRESENTATION (data->tags->data));
 	g_return_if_fail (CRIA_IS_THEME (g_list_last (data->tags)->data));
 	
@@ -479,7 +485,7 @@ cria_sax_0_0_1_element_theme_end (Sax001Data* data)
 	data->tags = g_list_remove (
 			data->tags,
 			g_list_last (data->tags)->data);
-	g_debug ("cria_sax_0_0_1_element_theme_end(): end");
+	cdebug("cria_sax_0_0_1_element_theme_end()", "end");
 }
 
 #warning "FIXME: do we have a cooler way to do reflections in C?"
@@ -496,31 +502,21 @@ static cria_tag_data cria_0_0_1_tag_data[] = {
 };
 
 static GHashTable*
-hash_table_from_strarray (gchar const **strarray)
-{
+hash_table_from_strarray(const gchar **strarray) {
 	GHashTable	* hash_table;
-	gchar	const	**iterator;
-	gchar	const	* key = NULL;
+	const gchar	**iterator;
 
-	hash_table = g_hash_table_new_full (
-			g_str_hash,
-			g_str_equal,
-			g_free,
-			g_free);
+	hash_table = g_hash_table_new_full(g_str_hash,
+					   g_str_equal,
+					   g_free,
+					   g_free);
 
-	for (iterator = strarray; iterator && *iterator; iterator++)
-	{
-		if (!key)
-			key = *iterator;
-		else
-		{
-			g_debug ("hash_table_from_strarray(%i): \"%s\" => \"%s\"", __LINE__, key, *iterator);
-			g_hash_table_insert (
-					hash_table,
-					g_strdup (key),
-					g_strdup (*iterator));
-			key = NULL;
-		}
+	for(iterator = strarray; iterator && *iterator && *(iterator+1); iterator += 2) {
+		gchar* key = g_strdup(*iterator);
+		gchar* val = g_strdup(*(iterator + 1));
+		
+		cdebug("hash_table_from_strarray()", "0x%x: 0x%x=\"%s\" => 0x%x\"%s\"", (uintptr_t)hash_table, (uintptr_t)key, key, (uintptr_t)val, val);
+		g_hash_table_insert(hash_table, key, val);
 	}
 
 	return hash_table;
@@ -531,77 +527,71 @@ cria_0_0_1_start_element(gpointer userdata, gchar const* tagname, const gchar**a
 	guint		  index = 0;
 	Sax001Data	* sax_data = (Sax001Data*) userdata;
 
-	g_debug("cria_0_0_1_start_element(): %d elements on the stack", g_list_length(sax_data->tags));
+	cdebug("cria_0_0_1_start_element()", "%d elements on the stack", g_list_length(sax_data->tags));
 
-	if (sax_data->error != NULL) {
+	if(sax_data->error != NULL) {
 		/* if we already have entered some invalid state the just do not try to make
 		 * more damage happen
 		 */
 		return;
 	}
 
-	if (sax_data->unknown == 0) {
+	if(sax_data->unknown == 0) {
 		/* We know where we are */
-		for (index = 0; cria_0_0_1_tag_data[index].tag_name; index++) {
-			if (strcmp (tagname, cria_0_0_1_tag_data[index].tag_name) == 0) {
+		for(index = 0; cria_0_0_1_tag_data[index].tag_name; index++) {
+			if(strcmp(tagname, cria_0_0_1_tag_data[index].tag_name) == 0) {
 				break;
 			}
 		}
 	}
 
-	if (sax_data->unknown > 0 || cria_0_0_1_tag_data[index].tag_name == NULL) {
-		g_debug ("cria_0_0_1_start_element(%i): unknown tag \"%s\" (Unknown depth: %d)", __LINE__, tagname, sax_data->unknown);
+	if(sax_data->unknown > 0 || cria_0_0_1_tag_data[index].tag_name == NULL) {
+		cdebug("cria_0_0_1_start_element()", "unknown tag \"%s\" (Unknown depth: %d)", tagname, sax_data->unknown);
 		sax_data->unknown++;
 	} else {
-		if (cria_0_0_1_tag_data[index].tag_start) {
+		if(cria_0_0_1_tag_data[index].tag_start) {
 			GHashTable	* attrs = NULL;
 
-			if (attributes != NULL) {
-				attrs = hash_table_from_strarray (attributes);
+			if(attributes != NULL) {
+				attrs = hash_table_from_strarray(attributes);
 			}
 			
-			cria_0_0_1_tag_data[index].tag_start (attrs, sax_data);
+			cria_0_0_1_tag_data[index].tag_start(attrs, sax_data);
+
+			if(attrs) {
+				g_hash_table_destroy(attrs);
+			}
 		} else {
-			g_debug ("cria_0_0_1_start_element(%i): skipping known tag \"%s\"", __LINE__, tagname);
+			cdebug("cria_0_0_1_start_element(%i)", "skipping known tag \"%s\"", tagname);
 		}
 	}
 }
 
 static void
-cria_0_0_1_end_element (
-		gpointer	  userdata,
-		gchar   const   * tagname)
-{
+cria_0_0_1_end_element(gpointer	userdata, gchar const* tagname) {
 	guint		  index;
 	Sax001Data      * sax_data;
 	
 	sax_data = (Sax001Data*) userdata;
 
-	if (sax_data->error != NULL) {
+	if(sax_data->error != NULL) {
 		/* ignore this (see explanation in cria_0_0_1_start_element) */
 		return;
 	}
 
-	if (sax_data->unknown > 0)
-	{
-		g_debug ("cria_0_0_1_end_element(%i): unknown tag \"%s\"", __LINE__, tagname);
+	if(sax_data->unknown > 0) {
+		cdebug("cria_0_0_1_end_element()", "unknown tag \"%s\"", tagname);
 		sax_data->unknown--;
-	}
-	else
-	{
-		for (index = 0; cria_0_0_1_tag_data[index].tag_name; index++)
-		{
-			if (strcmp (tagname, cria_0_0_1_tag_data[index].tag_name) == 0)
+	} else {
+		for(index = 0; cria_0_0_1_tag_data[index].tag_name; index++) {
+			if(strcmp(tagname, cria_0_0_1_tag_data[index].tag_name) == 0)
 				break;
 		}
 
-		if (cria_0_0_1_tag_data[index].tag_end)
-		{
+		if (cria_0_0_1_tag_data[index].tag_end) {
 			cria_0_0_1_tag_data[index].tag_end (sax_data);
-		}
-		else
-		{
-			g_debug ("cria_0_0_1_end_element(%i): skipping known tag \"%s\"", __LINE__, tagname);
+		} else {
+			cdebug("cria_0_0_1_end_element()", "skipping known tag \"%s\"", tagname);
 		}
 	}
 }
@@ -642,3 +632,4 @@ static xmlSAXHandler cria_0_0_1_handler = {
 	NULL, /* errorSAXFunc error;					*/
 	NULL  /* fatalErrorSAXFunc fatalError;				*/
 };
+
