@@ -3,7 +3,7 @@
  * AUTHORS
  *       Sven Herzberg        <herzi@gnome-de.org>
  *
- * Copyright (C) 2004 Sven Herzberg
+ * Copyright (C) 2004,2005 Sven Herzberg
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,9 @@
 #include <config.h>
 #endif
 
+#include "presentation.h"
+#include "presentation-priv.h"
+
 #include <inttypes.h>
 
 #include <string.h>
@@ -35,27 +38,27 @@
 #include <gsf-gnome/gsf-input-gnomevfs.h>
 #include <gsf/gsf-input-stdio.h>
 
+#include <cdebug.h>
+
 #include "application.h"
-#include "debug.h"
 #include "marshallers.h"
-#include "presentation.h"
 #include "presentation-parser.h"
 #include "theme.h"
 
-struct _CriaPresentationPrivate {
-	GnomeVFSURI	* uri;
-	GList		* slides;
-	GHashTable	* themes;
-};
+#define STR_UNSAVED_PRESENTATION _("Unsaved Presentation")
 
 enum {
 	PROP_0,
+	PROP_LOOP,
+	PROP_SLIDE_AUTO,
+	PROP_SLIDE_DELAY,
 	PROP_TITLE,
 	PROP_URI,
 };
 
 enum {
 	INSERTED_SLIDE_SIGNAL,
+	SLIDE_DELETED_SIGNAL,
 	N_SIGNALS
 };
 
@@ -76,6 +79,9 @@ static void		  cria_presentation_inserted_slide     (CriaPresentation* self,
 								gint		  new_position,
 								CriaSlide	* new_slide);
 
+static GObjectClass* parent_class = NULL;
+static gint unsaved_presentations = 0;
+
 /**
  * cria_presentation_add_theme:
  * @self: a CriaPresentation
@@ -87,83 +93,68 @@ void
 cria_presentation_add_theme(CriaPresentation* self, CriaTheme* theme) {
 	g_return_if_fail(CRIA_IS_PRESENTATION (self));
 	g_return_if_fail(CRIA_IS_THEME (theme));
-	g_return_if_fail(g_hash_table_lookup(self->priv->themes, cria_theme_get_name (theme)) == NULL);
+	g_return_if_fail(g_hash_table_lookup(self->themes, cria_theme_get_name (theme)) == NULL);
 
-	g_hash_table_insert(self->priv->themes,
+	g_hash_table_insert(self->themes,
 			    g_strdup(cria_theme_get_name(theme)),
 			    theme);
 }
 
+gboolean
+cria_presentation_get_auto_switch(CriaPresentation* self) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), FALSE);
+
+	return self->auto_switch;
+}
+
 /**
- * cria_presentation_append_slide:
+ * cria_presentation_get_default_name:
  * @self: a #CriaPresentation
- * @slide: a #CriaSlide
  *
- * Appends @slide to @presentation.
+ * Get a default filename for @self. This can be used from file choosers to
+ * suggest a filename for a presentation.
+ *
+ * Returns a suggested filename. Don't forget to g_free() it.
  */
-void
-cria_presentation_append_slide(CriaPresentation* self, CriaSlide* slide) {
-	g_return_if_fail(CRIA_IS_PRESENTATION(self));
-	g_return_if_fail(CRIA_IS_SLIDE(slide));
-
-	g_debug ("Presentation::appendSlide(): appending slide \"%s\" with index %i (dev-count)", cria_slide_get_title (slide), g_list_length (self->priv->slides));
-	
-	cria_presentation_insert_slide(self, slide, -1);
-}
-
-static void
-cria_presentation_class_init(CriaPresentationClass* cria_presentation_class) {
-	GObjectClass	* g_object_class;
-
-	/* setting up the presentation class */
-	cria_presentation_class->inserted_slide = cria_presentation_inserted_slide;
-
-	cria_presentation_signals[INSERTED_SLIDE_SIGNAL] = g_signal_new("inserted-slide",
-									CRIA_TYPE_PRESENTATION,
-									G_SIGNAL_RUN_LAST,
-									G_STRUCT_OFFSET(CriaPresentationClass,
-											inserted_slide),
-									NULL,
-									NULL,
-									cria_marshal_VOID__INT_OBJECT,
-									G_TYPE_NONE,
-									2,
-									G_TYPE_INT,
-									CRIA_TYPE_SLIDE);
-
-	/* setting up the gobject class */
-	g_object_class = G_OBJECT_CLASS(cria_presentation_class);
-
-	g_object_class->set_property = cria_presentation_set_property;
-	g_object_class->get_property = cria_presentation_get_property;
-
-	g_object_class_install_property(g_object_class,
-					PROP_TITLE,
-					g_param_spec_string("title",
-							    "Title",
-							    "The title of the presentation",
-							    _("untitled"),
-							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property(g_object_class,
-					PROP_URI,
-					g_param_spec_pointer("uri",
-							     "GnomeVFSURI for this file",
-							     "The URI this file can be located at, NULL if newly created",
-							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-}
-
-const gchar*
+gchar*
 cria_presentation_get_filename(CriaPresentation* self) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
-	g_assert(self->priv != NULL);
+	gchar* retval;
+	const gchar* title = cria_presentation_get_title(self);
 
-	if(self->priv->uri == NULL) {
-		return _("Unsaved Presentation");
+	if(cria_presentation_get_uri(self)) {
+		retval = gnome_vfs_uri_extract_short_name(cria_presentation_get_uri(self));
+	} else if(strcmp(STR_UNSAVED_PRESENTATION, title)) {
+		GString* string = g_string_new(title);
+		g_string_append(string, ".criawips");
+		retval = string->str;
+		g_string_free(string, FALSE);
 	} else {
-		GnomeVFSFileInfo* info = gnome_vfs_file_info_new();
-		gnome_vfs_get_file_info_uri(self->priv->uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
-		return info->name;
+#warning "getDefaultName(): FIXME: try to get some information from the first slide with text or similar"
+		retval = g_strdup_printf("%s.criawips", _("New Presentation"));
 	}
+
+	return retval;
+}
+
+gboolean
+cria_presentation_get_loop(CriaPresentation* self) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), FALSE);
+
+	return self->loop;
+}
+
+gdouble
+cria_presentation_get_slide_delay(CriaPresentation* self) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), 0.0);
+
+	return self->slide_delay;
+}
+
+GHashTable*
+cria_presentation_get_themes(CriaPresentation *self) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), NULL);
+	
+	return self->themes;
 }
 
 static void
@@ -173,53 +164,27 @@ cria_presentation_get_property(GObject* object, guint prop_id, GValue* value, GP
 	self = CRIA_PRESENTATION(object);
 
 	switch(prop_id) {
+	case PROP_LOOP:
+		g_value_set_boolean(value, cria_presentation_get_loop(self));
+		break;
+	case PROP_SLIDE_AUTO:
+		g_value_set_boolean(value, cria_presentation_get_auto_switch(self));
+		break;
+	case PROP_SLIDE_DELAY:
+		g_value_set_double(value, cria_presentation_get_slide_delay(self));
+		break;
 	case PROP_TITLE:
 		g_value_set_string(value, self->title);
 		break;
 	case PROP_URI:
-		g_value_set_pointer(value, self->priv->uri);
+		g_value_set_pointer(value, self->uri);
 		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
-						  prop_id,
-						  param_spec);
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, param_spec);
 		break;
 	}
 }
 
-CriaSlide*
-cria_presentation_get_slide (
-		CriaPresentation* self,
-		guint		  slide)
-{
-	CriaSlide	* retval;
-	
-	g_return_val_if_fail (CRIA_IS_PRESENTATION (self), NULL);
-	g_return_val_if_fail (slide < cria_presentation_n_slides (self), NULL);
-	
-	retval = CRIA_SLIDE (g_list_nth (self->priv->slides, slide)->data);
-
-	return retval;
-}
-
-/**
- * cria_presentation_get_slide_index:
- * @self: The presentation to search in
- * @slide: The slide to be found
- *
- * Get the index of a slide in a presentation.
- *
- * Returns the index of the slide if found, -1 otherwise
- */
-gint
-cria_presentation_get_slide_index(CriaPresentation* self, CriaSlide const* slide) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
-	g_assert(slide != NULL && CRIA_IS_SLIDE(slide));
-	g_assert(self->priv != NULL);
-	
-	return g_list_index(self->priv->slides, slide);
-}
-
 /**
  * cria_presentation_get_slide_size:
  * @self: The presentation to query
@@ -231,28 +196,18 @@ cria_presentation_get_slide_index(CriaPresentation* self, CriaSlide const* slide
  */
 const GoPoint*
 cria_presentation_get_slide_size(CriaPresentation* self) {
-#warning "Presentation::getSlideSize(): FIXME make this configurable and return the value of the presentation instead of a predfined one"
-	static GoPoint* size = NULL;
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), NULL);
 	
-	if(!size) {
-		size = g_new0(GoPoint,1);
-		size->x = 5760;
-		size->y = 4320;
-	};
+	cdebugo(self, "getSlideSlize()", "%llix%lli", self->size.x, self->size.y);
 
-	g_debug("Presentation:getSlideSlize(): %llix%lli", size->x, size->y);
-
-	return size;
+	return &(self->size);
 }
 
 CriaTheme*
-cria_presentation_get_theme (
-		CriaPresentation* self,
-		const	char	* theme)
-{
-	g_return_val_if_fail (CRIA_IS_PRESENTATION (self), NULL);
+cria_presentation_get_theme(CriaPresentation* self, const char* theme) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION (self), NULL);
 
-	return CRIA_THEME (g_hash_table_lookup (self->priv->themes, theme));
+	return CRIA_THEME(g_hash_table_lookup(self->themes, theme));
 }
 
 /**
@@ -261,103 +216,40 @@ cria_presentation_get_theme (
  *
  * Get the title of the presentation.
  *
- * Returns the title of the presentation, of one is set, %NULL otherwise
+ * Returns the title of the presentation, of one is set, an empty string otherwise.
  */
-const char*
+const gchar*
 cria_presentation_get_title(CriaPresentation* self) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), "");
+	g_return_val_if_fail(self->title, "");
 
-	g_debug("Presentation::getTitle(): %s", self->title);
-	
 	return self->title;
 }
 
-GType
-cria_presentation_get_type (void)
-{
-	static GType	type = 0;
-
-	if (!type)
-	{
-		const GTypeInfo info = {
-			sizeof (CriaPresentationClass),
-			NULL,	/* base initializer */
-			NULL,	/* base finalizer */
-			(GClassInitFunc)cria_presentation_class_init,
-			NULL,	/* class finalizer */
-			NULL,	/* class data */
-			sizeof (CriaPresentation),
-			0,
-			(GInstanceInitFunc)cria_presentation_init,
-			0
-		};
-
-		type = g_type_register_static (
-				G_TYPE_OBJECT,
-				"CriaPresentation",
-				&info,
-				0);
-	}
-
-	return type;
-}
-
 /**
  * cria_presentation_get_uri:
  * @self: the presentation to get the uri from
  *
  * Get information about the path of a presentation.
  *
- * Returns a GnomeVFSURI defining the path and filename of a presentation
+ * Returns a GnomeVFSURI defining the path and filename of a presentation,
+ * NULL if none is set. If you want to use it, don't forget to gnome_vfs_uri_ref()
  */
-const GnomeVFSURI*
+GnomeVFSURI*
 cria_presentation_get_uri(CriaPresentation* self) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
-	g_assert(self->priv != NULL);
-#warning "Presentation::getURI(): FIXME: ensure some valid URI"
-	g_assert(self->priv->uri != NULL);
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), NULL);
 	
-	return self->priv->uri;
-}
-
-
-static void
-cria_presentation_init (CriaPresentation *self)
-{
-	self->priv = g_new0 (CriaPresentationPrivate, 1);
-
-	self->priv->themes = g_hash_table_new_full (
-			g_str_hash,
-			g_str_equal,
-			g_free,
-			g_object_unref);
-}
-
-/**
- * cria_presentation_insert_slide:
- * @self: a #CriaPresentation
- * @slide: a #CriaSlide
- * @pos: a position
- *
- * Insert @slide at @pos in @self. If @pos is smaller than 0 or bigger than the
- * list's length, the slide will be appended.
- */
-void
-cria_presentation_insert_slide(CriaPresentation* self, CriaSlide* slide, gint pos) {
-	g_return_if_fail(CRIA_IS_PRESENTATION(self));
-	g_return_if_fail(CRIA_IS_SLIDE(slide));
-	g_return_if_fail(cria_presentation_get_slide_index(self, slide) == -1);
-	
-	self->priv->slides = g_list_insert(self->priv->slides, slide, pos);
-
-	pos = cria_presentation_get_slide_index(self, slide);
-	g_debug("Presentation::insertSlide(): new index = %i", pos);
-	g_signal_emit(self, cria_presentation_signals[INSERTED_SLIDE_SIGNAL], 0, pos, slide);
+	return self->uri;
 }
 
 static void
 cria_presentation_inserted_slide(CriaPresentation* self, gint new_position, CriaSlide* slide) {
-#warning "Presentation::insertedSlide(): set or unset the save state here"
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+
+	if(self->saved) {
+		self->saved = FALSE;
+		g_object_notify(G_OBJECT(self), "saved");
+	}
 }
 
 /**
@@ -369,13 +261,11 @@ cria_presentation_inserted_slide(CriaPresentation* self, gint new_position, Cria
  */
 CriaPresentation*
 cria_presentation_new_default(void) {
-	CriaPresentation* self  = CRIA_PRESENTATION(g_object_new(CRIA_TYPE_PRESENTATION, NULL));
+	CriaPresentation* self  = CRIA_PRESENTATION(g_object_new(CRIA_TYPE_PRESENTATION, "title", STR_UNSAVED_PRESENTATION, NULL));
 	CriaTheme	* theme = cria_theme_new(_("Default Theme"));
 	CriaSlide	* master_slide = cria_slide_new(NULL),
-			* slide = cria_slide_new(self);
+			* slide = cria_slide_new(CRIA_SLIDE_LIST(self));
 
-#warning "Presentation::newDefault(): set a GnomeVFSURI"
-#warning "Presentation::newDefault(): create a pool of unsaved presentations to be able handle 'Unsaved 1', 'Unsaved 2', etc."
 	cria_theme_add_master_slide(theme, master_slide);
 	cria_presentation_add_theme(self, theme);
 	cria_slide_set_theme(slide, theme);
@@ -397,33 +287,32 @@ cria_presentation_new_default(void) {
  */
 CriaPresentation*
 cria_presentation_new_from_file(const gchar* filename, GError** error) {
-	CriaPresentation* presentation;
+	CriaPresentation* self;
 	gchar		* uri	= NULL;
 
-	g_assert(filename != NULL && strlen(filename));
-	g_assert(*error == NULL);
+	g_return_val_if_fail(filename != NULL && strlen(filename), NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 	
-	g_debug("Presentation::newFromFile(): start");
+	cdebugt(CRIA_TYPE_PRESENTATION, "newFromFile()", "start");
 
 	uri = gnome_vfs_make_uri_from_shell_arg(filename);
-	presentation = cria_presentation_new_from_text_uri(uri, error);
+	self = cria_presentation_new_from_text_uri(uri, error);
 	
-	if(*error) {
-		if(presentation) {
-			g_object_unref(presentation);
+	if(error && *error) {
+		if(self) {
+			g_object_unref(self);
 		}
 		
-		presentation = NULL;
+		self = NULL;
 	} else {
 		cria_application_add_to_recent(uri);
-		g_debug("Presentation::newFromFile(): added uri '%s'", uri);
+		cdebugo(self, "newFromFile()", "added uri '%s'", uri);
 	}
 
-	g_debug("Presentation::newFromFile(): end");
+	cdebugo(self, "newFromFile()", "end");
 	
 	g_free(uri);
-	uri = NULL;
-	return presentation;
+	return self;
 }
 
 /**
@@ -442,8 +331,8 @@ cria_presentation_new_from_text_uri(const gchar* text_uri, GError**errloc) {
 	GnomeVFSURI	* uri	= NULL;
 	GnomeVFSFileInfo* info  = NULL;
 
-	g_assert(text_uri != NULL && strlen(text_uri));
-	g_assert(*errloc == NULL);
+	g_return_val_if_fail(text_uri != NULL && strlen(text_uri), NULL);
+	g_return_val_if_fail(errloc == NULL || *errloc == NULL, NULL);
 
 	uri = gnome_vfs_uri_new(text_uri);
 	info = gnome_vfs_file_info_new();
@@ -452,7 +341,7 @@ cria_presentation_new_from_text_uri(const gchar* text_uri, GError**errloc) {
 	presentation = cria_presentation_new_from_uri(uri, &error);
 	gnome_vfs_uri_unref(uri);
 
-	if(error) {
+	if(error && errloc) {
 		*errloc = error;
 		return NULL;
 	}
@@ -475,39 +364,71 @@ cria_presentation_new_from_uri(GnomeVFSURI* uri, GError** errloc) {
 	GError		* error = NULL;
 	CriaPresentation* self = NULL;
 
-	g_assert(uri != NULL);
-	g_assert(*errloc == NULL);
+	g_return_val_if_fail(uri != NULL, NULL);
+	g_return_val_if_fail(errloc == NULL || *errloc == NULL, NULL);
 
 	input = gsf_input_gnomevfs_new_uri(uri, &error);
 
-	if (error) {
-		*errloc = error;
-		return NULL;
+	if(!error) {
+#warning "newFromUri(): FIXME: add file type detection"
+		/* TODO add mime type detection:
+		 * this one works basically by getting a GnomeVFSFileInfo from the URI
+		 * and let GnomeVFS do the detection */
+#warning "newFromUri(): FIXME: extract file parsing into a plugin"
+		/* TODO extract the file reading into a plugin (refer to gnumeric's code
+		 * for examples) */
+		
+		self = cria_presentation_populate_from_xml_input(input, uri, &error);
 	}
 
-	/* TODO add mime type detection:
-	 * this one works basically by getting a GnomeVFSFileInfo from the URI
-	 * and let GnomeVFS do the detection */
-	/* TODO extract the file reading into a plugin (refer to gnumeric's code
-	 * for examples) */
-	
-	self = g_object_new(CRIA_TYPE_PRESENTATION, NULL);
-	cria_presentation_set_uri(self, uri);
-	cria_presentation_populate_from_xml_input(self, input, &error);
-
-	if (error) {
-		*errloc = error;
-		return NULL;
+	if(!error) {
+		return self;
+	} else if(self) {
+		g_object_unref(self);
 	}
-	
-	return self;
+
+	*errloc = error;
+	return NULL;
 }
 
-guint
-cria_presentation_n_slides(CriaPresentation* self) {
-	g_return_val_if_fail (CRIA_IS_PRESENTATION (self), 0);
-	
-	return g_list_length (self->priv->slides);
+void
+cria_presentation_set_auto_switch(CriaPresentation* self, gboolean auto_switch) {
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+
+	if(self->auto_switch == auto_switch) {
+		return;
+	}
+
+	self->auto_switch = auto_switch;
+
+	g_object_notify(G_OBJECT(self), "auto-switch");
+}
+
+void
+cria_presentation_set_loop(CriaPresentation* self, gboolean loop) {
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+
+	if(self->loop == loop) {
+		return;
+	}
+
+	self->loop = loop;
+
+	g_object_notify(G_OBJECT(self), "loop");
+}
+
+void
+cria_presentation_set_slide_delay(CriaPresentation* self, gdouble delay) {
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+
+	if(self->slide_delay == delay) {
+		return;
+	}
+
+	self->slide_delay = delay;
+	cdebugo(self, "setSlideDelay()", "new delay is %f", self->slide_delay);
+
+	g_object_notify(G_OBJECT(self), "slide-delay");
 }
 
 static void
@@ -517,6 +438,15 @@ cria_presentation_set_property(GObject* object, guint prop_id, const GValue* val
 	self = CRIA_PRESENTATION (object);
 	
 	switch (prop_id) {
+	case PROP_LOOP:
+		cria_presentation_set_loop(self, g_value_get_boolean(value));
+		break;
+	case PROP_SLIDE_AUTO:
+		cria_presentation_set_auto_switch(self, g_value_get_boolean(value));
+		break;
+	case PROP_SLIDE_DELAY:
+		cria_presentation_set_slide_delay(self, g_value_get_double(value));
+		break;
 	case PROP_TITLE:
 		cria_presentation_set_title(self, g_value_get_string(value));
 		break;
@@ -542,12 +472,12 @@ cria_presentation_set_property(GObject* object, guint prop_id, const GValue* val
  */
 void
 cria_presentation_set_title(CriaPresentation* self, const gchar* title) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+	g_return_if_fail(title != NULL);
 
-	g_debug("Presentation::setTitle(): setting to %s", title);
+	cdebugo(self, "setTitle()", "setting to %s", title);
 
-	if((title == NULL && self->title == NULL) ||
-	   (title != NULL && self->title != NULL && !strcmp(title, self->title))) {
+	if(title != NULL && self->title != NULL && !strcmp(title, self->title)) {
 		/* the text's are equal */
 		return;
 	}
@@ -556,7 +486,9 @@ cria_presentation_set_title(CriaPresentation* self, const gchar* title) {
 		g_free(self->title);
 	}
 
-	if(title) {
+	if(self->title == NULL && !strcmp(title, STR_UNSAVED_PRESENTATION)) {
+		self->title = g_strdup_printf("%s %d", STR_UNSAVED_PRESENTATION, ++unsaved_presentations);
+	} else {
 		self->title = g_strdup(title);
 	}
 
@@ -565,18 +497,21 @@ cria_presentation_set_title(CriaPresentation* self, const gchar* title) {
 
 void
 cria_presentation_set_uri(CriaPresentation* self, GnomeVFSURI* uri) {
-	g_assert(self != NULL && CRIA_IS_PRESENTATION(self));
-	g_assert(self->priv != NULL);
+	GnomeVFSURI* old;
+	
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
 	/* unfortunately we don't have GType information for GnomeVFSURI */
-	/* g_assert(self->priv->uri == NULL || GNOME_IS_VFS_URI(self->priv->uri)); */
-	/* g_assert(uri == NULL || GNOME_IS_VFS_URI(uri)); */
+	/* g_return_if_fail(self->uri == NULL || GNOME_IS_VFS_URI(self->uri)); */
+	/* g_return_if_fail(uri == NULL || GNOME_IS_VFS_URI(uri)); */
 
-	if(self->priv->uri) {
-		gnome_vfs_uri_unref(self->priv->uri);
-		self->priv->uri = NULL;
+	old = self->uri;
+
+	if(self->uri) {
+		gnome_vfs_uri_unref(self->uri);
+		self->uri = NULL;
 	}
 
-	self->priv->uri = uri;
+	self->uri = uri;
 	if(uri) {
 		gnome_vfs_uri_ref(uri);
 	}
@@ -584,3 +519,176 @@ cria_presentation_set_uri(CriaPresentation* self, GnomeVFSURI* uri) {
 	g_object_notify(G_OBJECT(self), "uri");
 }
 
+/* GObject stuff */
+static void
+cria_presentation_init(CriaPresentation *self) {
+	self->themes = g_hash_table_new_full(g_str_hash,
+					     g_str_equal,
+					     g_free,
+					     g_object_unref);
+	self->size.x = 5760;
+	self->size.y = 4320;
+}
+
+static void
+cp_finalize(GObject* obj) {
+	CriaPresentation* self = CRIA_PRESENTATION(obj);
+	
+	g_hash_table_destroy(self->themes);
+	self->themes = NULL;
+	
+	g_list_foreach(self->slides, (GFunc)g_object_unref, NULL);
+	
+	parent_class->finalize(obj);
+}
+
+static void
+cria_presentation_class_init(CriaPresentationClass* cria_presentation_class) {
+	GObjectClass	* g_object_class;
+
+	parent_class = g_type_class_peek_parent(cria_presentation_class);
+
+	/* setting up the gobject class */
+	g_object_class = G_OBJECT_CLASS(cria_presentation_class);
+
+	g_object_class->finalize     = cp_finalize;
+	g_object_class->set_property = cria_presentation_set_property;
+	g_object_class->get_property = cria_presentation_get_property;
+
+	g_object_class_install_property(g_object_class,
+					PROP_SLIDE_AUTO,
+					g_param_spec_boolean("auto-switch",
+							     "Switch between slides automatically",
+							     "Specifies whether this presentation should automatically switch between slides",
+							     FALSE,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property(g_object_class,
+					PROP_LOOP,
+					g_param_spec_boolean("loop",
+							     "Loop",
+							     "Restart presentation after the last slide",
+							     FALSE,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property(g_object_class,
+					PROP_SLIDE_DELAY,
+					g_param_spec_double("slide-delay",
+							    "Slide Delay",
+							    "The time that a slide is shown in auto-switch mode by default (in seconds)",
+							    0.0,
+							    G_MAXDOUBLE,
+							    5.0,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property(g_object_class,
+					PROP_TITLE,
+					g_param_spec_string("title",
+							    "Title",
+							    "The title of the presentation",
+							    _("untitled"),
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property(g_object_class,
+					PROP_URI,
+					g_param_spec_pointer("uri",
+							     "GnomeVFSURI for this file",
+							     "The URI this file can be located at, NULL if newly created",
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	/* setting up the presentation class */
+/*	cria_presentation_class->slide_deleted  = cp_slide_deleted;*/
+	cria_presentation_class->inserted_slide = cria_presentation_inserted_slide;
+
+	cria_presentation_signals[SLIDE_DELETED_SIGNAL] =
+				g_signal_new("slide-deleted",
+					     CRIA_TYPE_PRESENTATION,
+					     G_SIGNAL_RUN_LAST,
+					     G_STRUCT_OFFSET(CriaPresentationClass, slide_deleted),
+					     NULL, NULL,
+					     g_cclosure_marshal_VOID__INT,
+					     G_TYPE_NONE,
+					     1,
+					     G_TYPE_INT);
+	cria_presentation_signals[INSERTED_SLIDE_SIGNAL] = g_signal_new("inserted-slide",
+									CRIA_TYPE_PRESENTATION,
+									G_SIGNAL_RUN_LAST,
+									G_STRUCT_OFFSET(CriaPresentationClass,
+											inserted_slide),
+									NULL,
+									NULL,
+									cria_marshal_VOID__INT_OBJECT,
+									G_TYPE_NONE,
+									2,
+									G_TYPE_INT,
+									CRIA_TYPE_SLIDE);
+}
+
+static guint
+cp_n_slides(CriaSlideList* self) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), 0);
+	
+	return g_list_length(CRIA_PRESENTATION(self)->slides);
+}
+
+static CriaSlide*
+cp_get(CriaSlideList* self, guint slide) {
+	CriaSlide* retval;
+	
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), NULL);
+	g_return_val_if_fail(slide < cp_n_slides(self), NULL);
+	
+	retval = g_list_nth(CRIA_PRESENTATION(self)->slides, slide)->data;
+
+	g_return_val_if_fail(CRIA_IS_SLIDE(retval), NULL);
+
+	return retval;
+}
+
+static gint
+cp_index(CriaSlideList* self, const CriaSlide* slide) {
+	g_return_val_if_fail(CRIA_IS_PRESENTATION(self), -1);
+	g_return_val_if_fail(CRIA_IS_SLIDE(slide), -1);
+	
+	return g_list_index(CRIA_PRESENTATION(self)->slides, slide);
+}
+
+static void
+cp_insert(CriaSlideList* self, CriaSlide* slide, guint pos) {
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+	g_return_if_fail(CRIA_IS_SLIDE(slide));
+	g_return_if_fail(cp_index(self, slide) == -1);
+	
+	CRIA_PRESENTATION(self)->slides = g_list_insert(CRIA_PRESENTATION(self)->slides, slide, pos);
+
+	pos = cp_index(self, slide);
+	cdebugo(self, "insertSlide()", "new index = %i", pos);
+	
+#warning "inserSlide(): FIXME: the signal should come from the container"
+	g_signal_emit(self, cria_presentation_signals[INSERTED_SLIDE_SIGNAL], 0, pos, slide);
+}
+
+static void
+cp_remove(CriaSlideList* self, guint pos) {
+	CriaSlide* slide;
+	
+	g_return_if_fail(CRIA_IS_PRESENTATION(self));
+	g_return_if_fail(pos < cp_n_slides(self));
+
+	slide = cp_get(self, pos);
+	CRIA_PRESENTATION(self)->slides = g_list_delete_link(CRIA_PRESENTATION(self)->slides, g_list_nth(CRIA_PRESENTATION(self)->slides, pos));
+
+	g_signal_emit(self, cria_presentation_signals[SLIDE_DELETED_SIGNAL], 0, pos);
+	
+	g_object_unref(slide);
+}
+
+static void
+cp_container_init(gpointer interface) {
+	CriaSlideListIface* iface = interface;
+
+	iface->get      = cp_get;
+	iface->index    = cp_index;
+	iface->insert   = cp_insert;
+	iface->n_slides = cp_n_slides;
+	iface->remove   = cp_remove;
+}
+
+G_DEFINE_TYPE_WITH_CODE(CriaPresentation, cria_presentation, G_TYPE_OBJECT, G_IMPLEMENT_INTERFACE(CRIA_TYPE_SLIDE_CONTAINER, cp_container_init))
+
