@@ -3,7 +3,7 @@
  * AUTHORS
  *       Sven Herzberg        <herzi@gnome-de.org>
  *
- * Copyright (C) 2004 Sven Herzberg
+ * Copyright (C) 2004,2005 Sven Herzberg
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,34 +35,37 @@ G_BEGIN_DECLS
 
 typedef struct _CriaPresentation	CriaPresentation;
 typedef struct _CriaPresentationClass	CriaPresentationClass;
-typedef struct _CriaPresentationPrivate	CriaPresentationPrivate;
 
-#include "slide.h"
+G_END_DECLS
 
+#include <dom/cria-slide.h>
+
+G_BEGIN_DECLS
+
+GHashTable*       cria_presentation_get_themes	       (CriaPresentation *self);
 void		  cria_presentation_add_theme	       (CriaPresentation* self,
 							CriaTheme	* theme);
-void		  cria_presentation_append_slide       (CriaPresentation* self,
-							CriaSlide	* slide);
-const gchar	* cria_presentation_get_filename       (CriaPresentation* self);
+gboolean	  cria_presentation_get_auto_switch    (CriaPresentation* self);
+gchar*		  cria_presentation_get_filename       (CriaPresentation* self);
+gboolean	  cria_presentation_get_loop           (CriaPresentation* self);
 GType		  cria_presentation_get_type	       (void);
-const GnomeVFSURI* cria_presentation_get_uri	       (CriaPresentation* self);
-CriaSlide	* cria_presentation_get_slide	       (CriaPresentation* self,
-							guint		  index);
-gint		  cria_presentation_get_slide_index    (CriaPresentation* self,
-							CriaSlide const	* slide);
+GnomeVFSURI*      cria_presentation_get_uri	       (CriaPresentation* self);
+gdouble		  cria_presentation_get_slide_delay    (CriaPresentation* self);
 const GoPoint	* cria_presentation_get_slide_size     (CriaPresentation* self);
 CriaTheme	* cria_presentation_get_theme	       (CriaPresentation* self,
 							const char	* theme);
-const gchar	* cria_presentation_get_title	       (CriaPresentation* self);
-void		  cria_presentation_insert_slide       (CriaPresentation* self,
-							CriaSlide       * slide,
-							gint		  position);
-guint		  cria_presentation_n_slides	       (CriaPresentation*self);
+const gchar     * cria_presentation_get_title	       (CriaPresentation* self);
 CriaPresentation* cria_presentation_new_default	       (void);
 CriaPresentation* cria_presentation_new_from_file      (const	gchar	* filename,
 							GError		**error);
 CriaPresentation* cria_presentation_new_from_text_uri  (const gchar	* text_uri,
 							GError		**error);
+void		  cria_presentation_set_auto_switch    (CriaPresentation* self,
+							gboolean	  auto_switch);
+void		  cria_presentation_set_loop           (CriaPresentation* self,
+							gboolean	  loop);
+void		  cria_presentation_set_slide_delay    (CriaPresentation* self,
+							gdouble		  delay);
 void		  cria_presentation_set_title	       (CriaPresentation* self,
 							const char	* title);
 void		  cria_presentation_set_uri	       (CriaPresentation* self,
@@ -70,28 +73,11 @@ void		  cria_presentation_set_uri	       (CriaPresentation* self,
 
 #define CRIA_TYPE_PRESENTATION			(cria_presentation_get_type())
 #define CRIA_PRESENTATION(object)		(G_TYPE_CHECK_INSTANCE_CAST((object), CRIA_TYPE_PRESENTATION, CriaPresentation))
-#define CRIA_PRESENTATION_CLASS(klass)		(G_TYPE_CHACK_CLASS_CAST((klass), CRIA_TYPE_PRESENTATION, CriaPresentationClass))
+#define CRIA_PRESENTATION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), CRIA_TYPE_PRESENTATION, CriaPresentationClass))
 #define CRIA_IS_PRESENTATION(object)		(G_TYPE_CHECK_INSTANCE_TYPE((object), CRIA_TYPE_PRESENTATION))
 #define CRIA_IS_PRESENTATION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), CRIA_TYPE_PRESENTATION))
 #define CRIA_PRESENTATION_GET_CLASS(object)	(G_TYPE_INSTANCE_GET_CLASS((object), CRIA_TYPE_PRESENTATION, CriaPresentationClass))
 
-struct _CriaPresentation {
-	GObject			  base_instance;
-	/* GoPoint		* size = 5760x4320; */
-	/* gboolean		  saved; */
-	gchar			* title;
-	CriaPresentationPrivate	* priv;
-};
-
-struct _CriaPresentationClass {
-	GObjectClass	  base_class;
-
-	/* signals */
-	void (*inserted_slide)	       (CriaPresentation* self,
-					gint		  new_position,
-					CriaSlide	* new_slide);
-};
-
 G_END_DECLS
 
 #endif /* PRESENTATION_H */
