@@ -46,7 +46,7 @@ void		  cria_slide_show_set_slide	       (CriaSlideShow		* self,
 
 #define CRIA_TYPE_SLIDE_SHOW			(cria_slide_show_get_type ())
 #define CRIA_SLIDE_SHOW(object)			(G_TYPE_CHECK_INSTANCE_CAST((object), CRIA_TYPE_SLIDE_SHOW, CriaSlideShow))
-#define CRIA_SLIDE_SHOW_CLASS(klass)		(G_TYPE_CHACK_CLASS_CAST((klass), CRIA_TYPE_SLIDE_SHOW, CriaSlideShowClass))
+#define CRIA_SLIDE_SHOW_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), CRIA_TYPE_SLIDE_SHOW, CriaSlideShowClass))
 #define CRIA_IS_SLIDE_SHOW(object)		(G_TYPE_CHECK_INSTANCE_TYPE((object), CRIA_TYPE_SLIDE_SHOW))
 #define CRIA_IS_SLIDE_SHOW_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE((klass), CRIA_TYPE_SLIDE_SHOW))
 #define CRIA_SLIDE_SHOW_GET_CLASS(object)	(G_TYPE_INSTANCE_GET_CLASS((object), CRIA_TYPE_SLIDE_SHOW, CriaSlideShowClass))
